@@ -49,7 +49,7 @@ const char* const ARMv7DOpcode::s_optionName[8] = {
 };
 
 const char* const ARMv7DOpcode::s_shiftNames[4] = {
-    "lsl", "lsr", "asl", "ror"
+    "lsl", "lsr", "asr", "ror"
 };
 
 const char* const ARMv7DOpcode::s_specialRegisterNames[3] = { "sp", "lr", "pc" };
@@ -944,7 +944,7 @@ const char* ARMv7DOpcodeDataProcessingShiftedReg::format()
             appendSeparator();
             appendRegisterName(rm());
             appendSeparator();
-            appendUnsignedImmediate(immediate5());
+            appendImmShift(type(), immediate5());
 
             return m_formatBuffer;
         }
