@@ -147,8 +147,8 @@ inline long lroundf(float num) { return static_cast<long>(roundf(num)); }
 
 #endif
 
-#if COMPILER(GCC) && OS(QNX)
-// The stdlib on QNX doesn't contain long abs(long). See PR #104666.
+#if COMPILER(GCC) && OS(QNX) && _CPPLIB_VER < 640
+// The stdlib on QNX < 6.6 doesn't contain long abs(long). See PR #104666.
 inline long long abs(long num) { return labs(num); }
 #endif
 
