@@ -49,9 +49,9 @@ QT_BEGIN_NAMESPACE
 
 //TODO: Move docs (and inheritence) to real base when docs can propagate. Currently this pretends to be the base class!
 /*!
-    \qmlsignal QtQuick.Particles::Affector::onAffectParticles(Array particles, real dt)
+    \qmlsignal QtQuick.Particles::Affector::affectParticles(Array particles, real dt)
 
-    This handler is called when particles are selected to be affected. particles contains
+    This signal is emitted when particles are selected to be affected. particles contains
     an array of particle objects which can be directly manipulated.
 
     dt is the time since the last time it was affected. Use dt to normalize
@@ -59,6 +59,8 @@ QT_BEGIN_NAMESPACE
 
     Note that JavaScript is slower to execute, so it is not recommended to use this in
     high-volume particle systems.
+
+    The corresponding handler is \c onAffectParticles.
 */
 
 /*!
