@@ -45,8 +45,6 @@
 #include <QObject>
 #include <QPointF>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QQuickDirection : public QObject
@@ -56,13 +54,12 @@ public:
     explicit QQuickDirection(QObject *parent = 0);
 
     virtual const QPointF sample(const QPointF &from);
-signals:
+Q_SIGNALS:
 
-public slots:
+public Q_SLOTS:
 
 protected:
 };
 
 QT_END_NAMESPACE
-QT_END_HEADER
 #endif // VARYINGVECTOR_H
