@@ -44,8 +44,6 @@
 
 #include <private/qtquickparticlesglobal_p.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class Q_QUICKPARTICLES_PRIVATE_EXPORT QQuickParticlesModule
@@ -56,6 +54,4 @@ public:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QQUICKPARTICLESMODULE_H
