@@ -43,8 +43,6 @@
 #define ATTRACTORAFFECTOR_H
 #include "qquickparticleaffector_p.h"
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QQuickAttractorAffector : public QQuickParticleAffector
@@ -100,7 +98,7 @@ public:
         return m_proportionalToDistance;
     }
 
-signals:
+Q_SIGNALS:
 
     void strengthChanged(qreal arg);
 
@@ -112,12 +110,12 @@ signals:
 
     void proportionalToDistanceChanged(Proportion arg);
 
-public slots:
+public Q_SLOTS:
 void setStrength(qreal arg)
 {
     if (m_strength != arg) {
         m_strength = arg;
-        emit strengthChanged(arg);
+        Q_EMIT strengthChanged(arg);
     }
 }
 
@@ -125,7 +123,7 @@ void setPointX(qreal arg)
 {
     if (m_x != arg) {
         m_x = arg;
-        emit pointXChanged(arg);
+        Q_EMIT pointXChanged(arg);
     }
 }
 
@@ -133,14 +131,14 @@ void setPointY(qreal arg)
 {
     if (m_y != arg) {
         m_y = arg;
-        emit pointYChanged(arg);
+        Q_EMIT pointYChanged(arg);
     }
 }
 void setAffectedParameter(AffectableParameters arg)
 {
     if (m_physics != arg) {
         m_physics = arg;
-        emit affectedParameterChanged(arg);
+        Q_EMIT affectedParameterChanged(arg);
     }
 }
 
@@ -148,7 +146,7 @@ void setProportionalToDistance(Proportion arg)
 {
     if (m_proportionalToDistance != arg) {
         m_proportionalToDistance = arg;
-        emit proportionalToDistanceChanged(arg);
+        Q_EMIT proportionalToDistanceChanged(arg);
     }
 }
 
@@ -163,5 +161,4 @@ Proportion m_proportionalToDistance;
 };
 
 QT_END_NAMESPACE
-QT_END_HEADER
 #endif // ATTRACTORAFFECTOR_H
