@@ -45,6 +45,7 @@
 #include "private/qanimationgroupjob_p.h"
 #include "private/qanimationjobutil_p.h"
 #include "private/qqmlengine_p.h"
+#include "private/qqmlglobal_p.h"
 
 #define DEFAULT_TIMER_INTERVAL 16
 
@@ -54,12 +55,14 @@ QT_BEGIN_NAMESPACE
 Q_GLOBAL_STATIC(QThreadStorage<QQmlAnimationTimer *>, animationTimer)
 #endif
 
+DEFINE_BOOL_CONFIG_OPTION(animationTickDump, QML_ANIMATION_TICK_DUMP);
+
 QAnimationJobChangeListener::~QAnimationJobChangeListener()
 {
 }
 
 QQmlAnimationTimer::QQmlAnimationTimer() :
-    QAbstractAnimationTimer(), lastTick(0), lastDelta(0),
+    QAbstractAnimationTimer(), lastTick(0),
     currentAnimationIdx(0), insideTick(false),
     startAnimationPending(false), stopTimerPending(false),
     runningLeafAnimations(0)
@@ -103,7 +106,6 @@ void QQmlAnimationTimer::updateAnimationsTime(qint64 delta)
         return;
 
     lastTick += delta;
-    lastDelta = delta;
 
     //we make sure we only call update time if the time has actually changed
     //it might happen in some cases that the time doesn't change because events are delayed
@@ -116,6 +118,11 @@ void QQmlAnimationTimer::updateAnimationsTime(qint64 delta)
                           + (animation->direction() == QAbstractAnimationJob::Forward ? delta : -delta);
             animation->setCurrentTime(elapsed);
         }
+        if (animationTickDump()) {
+            qDebug() << "***** Dumping Animation Tree ***** ( tick:" << lastTick << "delta:" << delta << ")";
+            for (int i = 0; i < animations.count(); ++i)
+                qDebug() << animations.at(i);
+        }
         insideTick = false;
         currentAnimationIdx = 0;
     }
@@ -142,8 +149,7 @@ void QQmlAnimationTimer::startAnimations()
 {
     startAnimationPending = false;
     //force timer to update, which prevents large deltas for our newly added animations
-    if (!animations.isEmpty())
-        QUnifiedTimer::instance()->maybeUpdateAnimationsToCurrentTime();
+    QUnifiedTimer::instance()->maybeUpdateAnimationsToCurrentTime();
 
     //we transfer the waiting animations into the "really running" state
     animations += animationsToStart;
@@ -155,12 +161,12 @@ void QQmlAnimationTimer::startAnimations()
 void QQmlAnimationTimer::stopTimer()
 {
     stopTimerPending = false;
-    if (animations.isEmpty()) {
+    bool pendingStart = startAnimationPending && animationsToStart.size() > 0;
+    if (animations.isEmpty() && !pendingStart) {
         QUnifiedTimer::resumeAnimationTimer(this);
         QUnifiedTimer::stopAnimationTimer(this);
         // invalidate the start reference time
         lastTick = 0;
-        lastDelta = 0;
     }
 }
 
@@ -374,6 +380,7 @@ void QAbstractAnimationJob::setState(QAbstractAnimationJob::State newState)
         {
             // this ensures that the value is updated now that the animation is running
             if (oldState == Stopped) {
+                m_currentLoop = 0;
                 if (isTopLevel) {
                     // currentTime needs to be updated if pauseTimer is active
                     RETURN_IF_DELETED(QQmlAnimationTimer::ensureTimerUpdate());
@@ -649,6 +656,21 @@ void QAbstractAnimationJob::removeAnimationChangeListener(QAnimationJobChangeLis
     }
 }
 
+void QAbstractAnimationJob::debugAnimation(QDebug d) const
+{
+    d << "AbstractAnimationJob(" << hex << (void *) this << dec << ")" << "duration:" << duration();
+}
+
+QDebug operator<<(QDebug d, const QAbstractAnimationJob *job)
+{
+    if (!job) {
+        d << "AbstractAnimationJob(null)";
+        return d;
+    }
+    job->debugAnimation(d);
+    return d;
+}
+
 QT_END_NAMESPACE
 
 //#include "moc_qabstractanimation2_p.cpp"
