@@ -136,9 +136,11 @@ public:
     {
         _stackSlotForTemp.reserve(function->tempCount);
 
-        foreach (IR::BasicBlock *bb, function->basicBlocks) {
+        foreach (IR::BasicBlock *bb, function->basicBlocks()) {
+            if (bb->isRemoved())
+                continue;
             _currentBasicBlock = bb;
-            foreach (IR::Stmt *s, bb->statements)
+            foreach (IR::Stmt *s, bb->statements())
                 process(s);
         }
 
