@@ -78,6 +78,7 @@ Q_DECLARE_METATYPE(QList<int>)
 /*!
   \since 5.0
   \class QJSEngine
+  \reentrant
 
   \brief The QJSEngine class provides an environment for evaluating JavaScript code.
 
@@ -234,7 +235,7 @@ void QJSEngine::collectGarbage()
     Evaluates \a program, using \a lineNumber as the base line number,
     and returns the result of the evaluation.
 
-    The script code will be evaluated in the current context.
+    The script code will be evaluated in the context of the global object.
 
     The evaluation of \a program can cause an exception in the
     engine; in this case the return value will be the exception
@@ -260,8 +261,11 @@ void QJSEngine::collectGarbage()
 */
 QJSValue QJSEngine::evaluate(const QString& program, const QString& fileName, int lineNumber)
 {
-    QV4::Scope scope(d->m_v4Engine);
-    QV4::ExecutionContext *ctx = d->m_v4Engine->currentContext();
+    QV4::ExecutionEngine *v4 = d->m_v4Engine;
+    QV4::Scope scope(v4);
+    QV4::ExecutionContext *ctx = v4->currentContext();
+    if (ctx != v4->rootContext)
+        ctx = v4->pushGlobalContext();
     QV4::ScopedValue result(scope);
 
     QV4::Script script(ctx, program, fileName, lineNumber);
@@ -272,7 +276,9 @@ QJSValue QJSEngine::evaluate(const QString& program, const QString& fileName, in
         result = script.run();
     if (scope.engine->hasException)
         result = ctx->catchException();
-    return new QJSValuePrivate(d->m_v4Engine, result);
+    if (ctx != v4->rootContext)
+        v4->popContext();
+    return new QJSValuePrivate(v4, result);
 }
 
 /*!
@@ -330,7 +336,11 @@ QJSValue QJSEngine::newQObject(QObject *object)
     Q_D(QJSEngine);
     QV4::ExecutionEngine *v4 = QV8Engine::getV4(d);
     QV4::Scope scope(v4);
-    QQmlEngine::setObjectOwnership(object, QQmlEngine::JavaScriptOwnership);
+    if (object) {
+        QQmlData *ddata = QQmlData::get(object, true);
+        if (!ddata || !ddata->explicitIndestructibleSet)
+            QQmlEngine::setObjectOwnership(object, QQmlEngine::JavaScriptOwnership);
+    }
     QV4::ScopedValue v(scope, QV4::QObjectWrapper::wrap(v4, object));
     return new QJSValuePrivate(v4, v);
 }
@@ -387,7 +397,7 @@ bool QJSEngine::convertV2(const QJSValue &value, int type, void *ptr)
             *reinterpret_cast<QString*>(ptr) = vp->string;
             return true;
         }
-        double d = QV4::__qmljs_string_to_number(vp->string);
+        double d = QV4::RuntimeHelpers::stringToNumber(vp->string);
         switch (type) {
             case QMetaType::Int:
                 *reinterpret_cast<int*>(ptr) = QV4::Primitive::toInt32(d);
