@@ -51,7 +51,6 @@
 #include <cmath>
 #include <qmath.h>
 #include <qnumeric.h>
-#include <cassert>
 #include <time.h>
 
 #include <private/qqmljsengine_p.h>
@@ -157,7 +156,7 @@ static inline bool InLeapYear(double t)
     if (x == 365)
         return 0;
 
-    assert(x == 366);
+    Q_ASSERT(x == 366);
     return 1;
 }
 
@@ -628,12 +627,13 @@ static double getLocalTZA()
 #ifndef Q_OS_WIN
     struct tm t;
     time_t curr;
+    tzset();
     time(&curr);
     localtime_r(&curr, &t);
     time_t locl = mktime(&t);
     gmtime_r(&curr, &t);
     time_t globl = mktime(&t);
-    return double(locl - globl) * 1000.0;
+    return (double(locl) - double(globl)) * 1000.0;
 #else
     TIME_ZONE_INFORMATION tzInfo;
     GetTimeZoneInformation(&tzInfo);
@@ -641,13 +641,12 @@ static double getLocalTZA()
 #endif
 }
 
-DEFINE_MANAGED_VTABLE(DateObject);
+DEFINE_OBJECT_VTABLE(DateObject);
 
 DateObject::DateObject(ExecutionEngine *engine, const QDateTime &date)
     : Object(engine->dateClass)
 {
-    setVTable(&static_vtbl);
-    type = Type_DateObject;
+    setVTable(staticVTable());
     value.setDouble(date.isValid() ? date.toMSecsSinceEpoch() : qSNaN());
 }
 
@@ -656,12 +655,12 @@ QDateTime DateObject::toQDateTime() const
     return ToDateTime(value.asDouble(), Qt::LocalTime);
 }
 
-DEFINE_MANAGED_VTABLE(DateCtor);
+DEFINE_OBJECT_VTABLE(DateCtor);
 
 DateCtor::DateCtor(ExecutionContext *scope)
     : FunctionObject(scope, QStringLiteral("Date"))
 {
-    setVTable(&static_vtbl);
+    setVTable(staticVTable());
 }
 
 ReturnedValue DateCtor::construct(Managed *m, CallData *callData)
@@ -677,7 +676,7 @@ ReturnedValue DateCtor::construct(Managed *m, CallData *callData)
         if (DateObject *d = arg->asDateObject())
             arg = d->value;
         else
-            arg = __qmljs_to_primitive(arg, PREFERREDTYPE_HINT);
+            arg = RuntimeHelpers::toPrimitive(arg, PREFERREDTYPE_HINT);
 
         if (arg->isString())
             t = ParseString(arg->stringValue()->toQString());
@@ -1308,8 +1307,8 @@ ReturnedValue DatePrototype::method_toISOString(CallContext *ctx)
 ReturnedValue DatePrototype::method_toJSON(CallContext *ctx)
 {
     Scope scope(ctx);
-    ScopedValue O(scope, __qmljs_to_object(ctx, ValueRef(&ctx->callData->thisObject)));
-    ScopedValue tv(scope, __qmljs_to_primitive(O, NUMBER_HINT));
+    ScopedValue O(scope, RuntimeHelpers::toObject(ctx, ValueRef(&ctx->callData->thisObject)));
+    ScopedValue tv(scope, RuntimeHelpers::toPrimitive(O, NUMBER_HINT));
 
     if (tv->isNumber() && !std::isfinite(tv->toNumber()))
         return Encode::null();
