@@ -44,6 +44,7 @@
 #include <QHash>
 #include <QVector>
 #include "qv4global_p.h"
+#include <private/qqmljsmemorypool_p.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -213,7 +214,7 @@ struct InternalClassTransition
 };
 uint qHash(const QV4::InternalClassTransition &t, uint = 0);
 
-struct InternalClass {
+struct InternalClass : public QQmlJS::Managed {
     ExecutionEngine *engine;
     Object *prototype;
     const ManagedVTable *vtable;
@@ -233,10 +234,13 @@ struct InternalClass {
     static InternalClass *create(ExecutionEngine *engine, const ManagedVTable *vtable, Object *proto);
     InternalClass *changePrototype(Object *proto);
     InternalClass *changeVTable(const ManagedVTable *vt);
+    static void addMember(Object *object, StringRef string, PropertyAttributes data, uint *index);
+    static void addMember(Object *object, String *string, PropertyAttributes data, uint *index);
     InternalClass *addMember(StringRef string, PropertyAttributes data, uint *index = 0);
     InternalClass *addMember(String *string, PropertyAttributes data, uint *index = 0);
     InternalClass *changeMember(String *string, PropertyAttributes data, uint *index = 0);
-    void removeMember(Object *object, Identifier *id);
+    static void changeMember(Object *object, String *string, PropertyAttributes data, uint *index = 0);
+    static void removeMember(Object *object, Identifier *id);
     uint find(const StringRef string);
     uint find(const String *s);
 
@@ -244,14 +248,19 @@ struct InternalClass {
     InternalClass *frozen();
 
     void destroy();
-    void markObjects();
 
 private:
+    InternalClass *addMemberImpl(String *string, PropertyAttributes data, uint *index);
     friend struct ExecutionEngine;
     InternalClass(ExecutionEngine *engine);
     InternalClass(const InternalClass &other);
 };
 
+struct InternalClassPool : public QQmlJS::MemoryPool
+{
+    void markObjects(ExecutionEngine *engine);
+};
+
 }
 
 QT_END_NAMESPACE
