@@ -46,7 +46,8 @@ QT_BEGIN_NAMESPACE
 
 using namespace QV4;
 
-Property *Lookup::lookup(Object *obj, PropertyAttributes *attrs)
+
+ReturnedValue Lookup::lookup(ValueRef thisObject, Object *obj, PropertyAttributes *attrs)
 {
     int i = 0;
     while (i < Size && obj) {
@@ -56,7 +57,7 @@ Property *Lookup::lookup(Object *obj, PropertyAttributes *attrs)
         if (index != UINT_MAX) {
             level = i;
             *attrs = obj->internalClass->propertyData.at(index);
-            return obj->memberData + index;
+            return !attrs->isAccessor() ? obj->memberData[index].asReturnedValue() : obj->getValue(thisObject, obj->propertyAt(index), *attrs);
         }
 
         obj = obj->prototype();
@@ -68,14 +69,173 @@ Property *Lookup::lookup(Object *obj, PropertyAttributes *attrs)
         index = obj->internalClass->find(name);
         if (index != UINT_MAX) {
             *attrs = obj->internalClass->propertyData.at(index);
-            return obj->memberData + index;
+            return !attrs->isAccessor() ? obj->memberData[index].asReturnedValue() : obj->getValue(thisObject, obj->propertyAt(index), *attrs);
         }
 
         obj = obj->prototype();
     }
-    return 0;
+    return Primitive::emptyValue().asReturnedValue();
 }
 
+ReturnedValue Lookup::lookup(Object *obj, PropertyAttributes *attrs)
+{
+    Object *thisObject = obj;
+    int i = 0;
+    while (i < Size && obj) {
+        classList[i] = obj->internalClass;
+
+        index = obj->internalClass->find(name);
+        if (index != UINT_MAX) {
+            level = i;
+            *attrs = obj->internalClass->propertyData.at(index);
+            return !attrs->isAccessor() ? obj->memberData[index].asReturnedValue() : thisObject->getValue(obj->propertyAt(index), *attrs);
+        }
+
+        obj = obj->prototype();
+        ++i;
+    }
+    level = Size;
+
+    while (obj) {
+        index = obj->internalClass->find(name);
+        if (index != UINT_MAX) {
+            *attrs = obj->internalClass->propertyData.at(index);
+            return !attrs->isAccessor() ? obj->memberData[index].asReturnedValue() : thisObject->getValue(obj->propertyAt(index), *attrs);
+        }
+
+        obj = obj->prototype();
+    }
+    return Primitive::emptyValue().asReturnedValue();
+}
+
+ReturnedValue Lookup::indexedGetterGeneric(Lookup *l, const ValueRef object, const ValueRef index)
+{
+    if (object->isObject() && index->asArrayIndex() < UINT_MAX) {
+        l->indexedGetter = indexedGetterObjectInt;
+        return indexedGetterObjectInt(l, object, index);
+    }
+    return indexedGetterFallback(l, object, index);
+}
+
+ReturnedValue Lookup::indexedGetterFallback(Lookup *l, const ValueRef object, const ValueRef index)
+{
+    Q_UNUSED(l);
+    ExecutionContext *ctx = l->engine->currentContext();
+    Scope scope(ctx);
+    uint idx = index->asArrayIndex();
+
+    Scoped<Object> o(scope, object);
+    if (!o) {
+        if (idx < UINT_MAX) {
+            if (String *str = object->asString()) {
+                if (idx >= (uint)str->toQString().length()) {
+                    return Encode::undefined();
+                }
+                const QString s = str->toQString().mid(idx, 1);
+                return scope.engine->newString(s)->asReturnedValue();
+            }
+        }
+
+        if (object->isNullOrUndefined()) {
+            QString message = QStringLiteral("Cannot read property '%1' of %2").arg(index->toQStringNoThrow()).arg(object->toQStringNoThrow());
+            return ctx->throwTypeError(message);
+        }
+
+        o = RuntimeHelpers::convertToObject(ctx, object);
+        if (!o) // type error
+            return Encode::undefined();
+    }
+
+    if (idx < UINT_MAX) {
+        if (!o->arrayData->hasAttributes()) {
+            ScopedValue v(scope, o->arrayData->get(idx));
+            if (!v->isEmpty())
+                return v->asReturnedValue();
+        }
+
+        return o->getIndexed(idx);
+    }
+
+    ScopedString name(scope, index->toString(ctx));
+    if (scope.hasException())
+        return Encode::undefined();
+    return o->get(name);
+
+}
+
+
+ReturnedValue Lookup::indexedGetterObjectInt(Lookup *l, const ValueRef object, const ValueRef index)
+{
+    uint idx = index->asArrayIndex();
+    if (idx == UINT_MAX || !object->isObject())
+        return indexedGetterGeneric(l, object, index);
+
+    Object *o = object->objectValue();
+    if (o->arrayData && o->arrayData->type == ArrayData::Simple) {
+        if (idx < static_cast<SimpleArrayData *>(o->arrayData)->len)
+            if (!o->arrayData->data[idx].isEmpty())
+                return o->arrayData->data[idx].asReturnedValue();
+    }
+
+    return indexedGetterFallback(l, object, index);
+}
+
+void Lookup::indexedSetterGeneric(Lookup *l, const ValueRef object, const ValueRef index, const ValueRef v)
+{
+    if (object->isObject()) {
+        Object *o = object->objectValue();
+        if (o->arrayData && o->arrayData->type == ArrayData::Simple && index->asArrayIndex() < UINT_MAX) {
+            l->indexedSetter = indexedSetterObjectInt;
+            indexedSetterObjectInt(l, object, index, v);
+            return;
+        }
+    }
+    indexedSetterFallback(l, object, index, v);
+}
+
+void Lookup::indexedSetterFallback(Lookup *l, const ValueRef object, const ValueRef index, const ValueRef value)
+{
+    ExecutionContext *ctx = l->engine->currentContext();
+    Scope scope(ctx);
+    ScopedObject o(scope, object->toObject(ctx));
+    if (scope.engine->hasException)
+        return;
+
+    uint idx = index->asArrayIndex();
+    if (idx < UINT_MAX) {
+        if (o->arrayData && o->arrayData->type == ArrayData::Simple) {
+            SimpleArrayData *s = static_cast<SimpleArrayData *>(o->arrayData);
+            if (s && idx < s->len && !s->data[idx].isEmpty()) {
+                s->data[idx] = value;
+                return;
+            }
+        }
+        o->putIndexed(idx, value);
+        return;
+    }
+
+    ScopedString name(scope, index->toString(ctx));
+    o->put(name, value);
+}
+
+void Lookup::indexedSetterObjectInt(Lookup *l, const ValueRef object, const ValueRef index, const ValueRef v)
+{
+    uint idx = index->asArrayIndex();
+    if (idx == UINT_MAX || !object->isObject()) {
+        indexedSetterGeneric(l, object, index, v);
+        return;
+    }
+
+    Object *o = object->objectValue();
+    if (o->arrayData && o->arrayData->type == ArrayData::Simple) {
+        SimpleArrayData *s = static_cast<SimpleArrayData *>(o->arrayData);
+        if (idx < s->len && !s->data[idx].isEmpty()) {
+            s->data[idx] = v;
+            return;
+        }
+    }
+    indexedSetterFallback(l, object, index, v);
+}
 
 ReturnedValue Lookup::getterGeneric(QV4::Lookup *l, const ValueRef object)
 {
@@ -106,8 +266,8 @@ ReturnedValue Lookup::getterGeneric(QV4::Lookup *l, const ValueRef object)
     }
 
     PropertyAttributes attrs;
-    Property *p = l->lookup(proto, &attrs);
-    if (p) {
+    ReturnedValue v = l->lookup(object, proto, &attrs);
+    if (v != Primitive::emptyValue().asReturnedValue()) {
         l->type = object->type();
         l->proto = proto;
         if (attrs.isData()) {
@@ -115,19 +275,65 @@ ReturnedValue Lookup::getterGeneric(QV4::Lookup *l, const ValueRef object)
                 l->getter = Lookup::primitiveGetter0;
             else if (l->level == 1)
                 l->getter = Lookup::primitiveGetter1;
-            return p->value.asReturnedValue();
+            return v;
         } else {
             if (l->level == 0)
                 l->getter = Lookup::primitiveGetterAccessor0;
             else if (l->level == 1)
                 l->getter = Lookup::primitiveGetterAccessor1;
-            return proto->getValue(object, p, attrs);
+            return v;
         }
     }
 
     return Encode::undefined();
 }
 
+ReturnedValue Lookup::getterTwoClasses(Lookup *l, const ValueRef object)
+{
+    Lookup l1 = *l;
+
+    if (l1.getter == Lookup::getter0 || l1.getter == Lookup::getter1) {
+        if (Object *o = object->asObject()) {
+            ReturnedValue v = o->getLookup(l);
+            Lookup l2 = *l;
+
+            if (l2.getter == Lookup::getter0 || l2.getter == Lookup::getter1) {
+                // if we have a getter0, make sure it comes first
+                if (l2.getter == Lookup::getter0)
+                    qSwap(l1, l2);
+
+                l->classList[0] = l1.classList[0];
+                l->classList[1] = l1.classList[1];
+                l->classList[2] = l2.classList[0];
+                l->classList[3] = l2.classList[1];
+                l->index = l1.index;
+                l->index2 = l2.index;
+
+                if (l1.getter == Lookup::getter0) {
+                    l->getter = (l2.getter == Lookup::getter0) ? Lookup::getter0getter0 : Lookup::getter0getter1;
+                } else {
+                    Q_ASSERT(l1.getter == Lookup::getter1 && l2.getter == Lookup::getter1);
+                    l->getter = Lookup::getter1getter1;
+                }
+                return v;
+            }
+        }
+    }
+
+    l->getter = getterFallback;
+    return getterFallback(l, object);
+}
+
+ReturnedValue Lookup::getterFallback(Lookup *l, const ValueRef object)
+{
+    QV4::Scope scope(l->name->engine());
+    QV4::ScopedObject o(scope, object->toObject(scope.engine->currentContext()));
+    if (!o)
+        return Encode::undefined();
+    QV4::ScopedString s(scope, l->name);
+    return o->get(s);
+}
+
 ReturnedValue Lookup::getter0(Lookup *l, const ValueRef object)
 {
     if (object->isManaged()) {
@@ -135,10 +341,9 @@ ReturnedValue Lookup::getter0(Lookup *l, const ValueRef object)
         // the internal class won't match
         Object *o = object->objectValue();
         if (l->classList[0] == o->internalClass)
-            return static_cast<Object *>(o)->memberData[l->index].value.asReturnedValue();
+            return o->memberData[l->index].asReturnedValue();
     }
-    l->getter = getterGeneric;
-    return getterGeneric(l, object);
+    return getterTwoClasses(l, object);
 }
 
 ReturnedValue Lookup::getter1(Lookup *l, const ValueRef object)
@@ -149,10 +354,9 @@ ReturnedValue Lookup::getter1(Lookup *l, const ValueRef object)
         Object *o = object->objectValue();
         if (l->classList[0] == o->internalClass &&
             l->classList[1] == o->prototype()->internalClass)
-            return o->prototype()->memberData[l->index].value.asReturnedValue();
+            return o->prototype()->memberData[l->index].asReturnedValue();
     }
-    l->getter = getterGeneric;
-    return getterGeneric(l, object);
+    return getterTwoClasses(l, object);
 }
 
 ReturnedValue Lookup::getter2(Lookup *l, const ValueRef object)
@@ -166,14 +370,64 @@ ReturnedValue Lookup::getter2(Lookup *l, const ValueRef object)
             if (l->classList[1] == o->internalClass) {
                 o = o->prototype();
                 if (l->classList[2] == o->internalClass)
-                    return o->memberData[l->index].value.asReturnedValue();
+                    return o->memberData[l->index].asReturnedValue();
             }
         }
     }
-    l->getter = getterGeneric;
-    return getterGeneric(l, object);
+    l->getter = getterFallback;
+    return getterFallback(l, object);
 }
 
+ReturnedValue Lookup::getter0getter0(Lookup *l, const ValueRef object)
+{
+    if (object->isManaged()) {
+        // we can safely cast to a QV4::Object here. If object is actually a string,
+        // the internal class won't match
+        Object *o = object->objectValue();
+        if (l->classList[0] == o->internalClass)
+            return o->memberData[l->index].asReturnedValue();
+        if (l->classList[2] == o->internalClass)
+            return o->memberData[l->index2].asReturnedValue();
+    }
+    l->getter = getterFallback;
+    return getterFallback(l, object);
+}
+
+ReturnedValue Lookup::getter0getter1(Lookup *l, const ValueRef object)
+{
+    if (object->isManaged()) {
+        // we can safely cast to a QV4::Object here. If object is actually a string,
+        // the internal class won't match
+        Object *o = object->objectValue();
+        if (l->classList[0] == o->internalClass)
+            return o->memberData[l->index].asReturnedValue();
+        if (l->classList[2] == o->internalClass &&
+            l->classList[3] == o->prototype()->internalClass)
+            return o->prototype()->memberData[l->index2].asReturnedValue();
+    }
+    l->getter = getterFallback;
+    return getterFallback(l, object);
+}
+
+ReturnedValue Lookup::getter1getter1(Lookup *l, const ValueRef object)
+{
+    if (object->isManaged()) {
+        // we can safely cast to a QV4::Object here. If object is actually a string,
+        // the internal class won't match
+        Object *o = object->objectValue();
+        if (l->classList[0] == o->internalClass &&
+            l->classList[1] == o->prototype()->internalClass)
+            return o->prototype()->memberData[l->index].asReturnedValue();
+        if (l->classList[2] == o->internalClass &&
+            l->classList[3] == o->prototype()->internalClass)
+            return o->prototype()->memberData[l->index2].asReturnedValue();
+        return getterFallback(l, object);
+    }
+    l->getter = getterFallback;
+    return getterFallback(l, object);
+}
+
+
 ReturnedValue Lookup::getterAccessor0(Lookup *l, const ValueRef object)
 {
     if (object->isManaged()) {
@@ -182,7 +436,7 @@ ReturnedValue Lookup::getterAccessor0(Lookup *l, const ValueRef object)
         Object *o = object->objectValue();
         if (l->classList[0] == o->internalClass) {
             Scope scope(o->engine());
-            FunctionObject *getter = o->memberData[l->index].getter();
+            FunctionObject *getter = o->propertyAt(l->index)->getter();
             if (!getter)
                 return Encode::undefined();
 
@@ -191,8 +445,8 @@ ReturnedValue Lookup::getterAccessor0(Lookup *l, const ValueRef object)
             return getter->call(callData);
         }
     }
-    l->getter = getterGeneric;
-    return getterGeneric(l, object);
+    l->getter = getterFallback;
+    return getterFallback(l, object);
 }
 
 ReturnedValue Lookup::getterAccessor1(Lookup *l, const ValueRef object)
@@ -204,7 +458,7 @@ ReturnedValue Lookup::getterAccessor1(Lookup *l, const ValueRef object)
         if (l->classList[0] == o->internalClass &&
             l->classList[1] == o->prototype()->internalClass) {
             Scope scope(o->engine());
-            FunctionObject *getter = o->prototype()->memberData[l->index].getter();
+            FunctionObject *getter = o->prototype()->propertyAt(l->index)->getter();
             if (!getter)
                 return Encode::undefined();
 
@@ -213,8 +467,8 @@ ReturnedValue Lookup::getterAccessor1(Lookup *l, const ValueRef object)
             return getter->call(callData);
         }
     }
-    l->getter = getterGeneric;
-    return getterGeneric(l, object);
+    l->getter = getterFallback;
+    return getterFallback(l, object);
 }
 
 ReturnedValue Lookup::getterAccessor2(Lookup *l, const ValueRef object)
@@ -229,7 +483,7 @@ ReturnedValue Lookup::getterAccessor2(Lookup *l, const ValueRef object)
                 o = o->prototype();
                 if (l->classList[2] == o->internalClass) {
                     Scope scope(o->engine());
-                    FunctionObject *getter = o->memberData[l->index].getter();
+                    FunctionObject *getter = o->propertyAt(l->index)->getter();
                     if (!getter)
                         return Encode::undefined();
 
@@ -240,17 +494,16 @@ ReturnedValue Lookup::getterAccessor2(Lookup *l, const ValueRef object)
             }
         }
     }
-    l->getter = getterGeneric;
-    return getterGeneric(l, object);
+    l->getter = getterFallback;
+    return getterFallback(l, object);
 }
 
-
 ReturnedValue Lookup::primitiveGetter0(Lookup *l, const ValueRef object)
 {
     if (object->type() == l->type) {
         Object *o = l->proto;
         if (l->classList[0] == o->internalClass)
-            return o->memberData[l->index].value.asReturnedValue();
+            return o->memberData[l->index].asReturnedValue();
     }
     l->getter = getterGeneric;
     return getterGeneric(l, object);
@@ -262,7 +515,7 @@ ReturnedValue Lookup::primitiveGetter1(Lookup *l, const ValueRef object)
         Object *o = l->proto;
         if (l->classList[0] == o->internalClass &&
             l->classList[1] == o->prototype()->internalClass)
-            return o->prototype()->memberData[l->index].value.asReturnedValue();
+            return o->prototype()->memberData[l->index].asReturnedValue();
     }
     l->getter = getterGeneric;
     return getterGeneric(l, object);
@@ -274,7 +527,7 @@ ReturnedValue Lookup::primitiveGetterAccessor0(Lookup *l, const ValueRef object)
         Object *o = l->proto;
         if (l->classList[0] == o->internalClass) {
             Scope scope(o->engine());
-            FunctionObject *getter = o->memberData[l->index].getter();
+            FunctionObject *getter = o->propertyAt(l->index)->getter();
             if (!getter)
                 return Encode::undefined();
 
@@ -294,7 +547,7 @@ ReturnedValue Lookup::primitiveGetterAccessor1(Lookup *l, const ValueRef object)
         if (l->classList[0] == o->internalClass &&
             l->classList[1] == o->prototype()->internalClass) {
             Scope scope(o->engine());
-            FunctionObject *getter = o->prototype()->memberData[l->index].getter();
+            FunctionObject *getter = o->prototype()->propertyAt(l->index)->getter();
             if (!getter)
                 return Encode::undefined();
 
@@ -316,13 +569,22 @@ ReturnedValue Lookup::stringLengthGetter(Lookup *l, const ValueRef object)
     return getterGeneric(l, object);
 }
 
+ReturnedValue Lookup::arrayLengthGetter(Lookup *l, const ValueRef object)
+{
+    if (ArrayObject *a = object->asArrayObject())
+        return a->memberData[ArrayObject::LengthPropertyIndex].asReturnedValue();
+
+    l->getter = getterGeneric;
+    return getterGeneric(l, object);
+}
+
 
 ReturnedValue Lookup::globalGetterGeneric(Lookup *l, ExecutionContext *ctx)
 {
     Object *o = ctx->engine->globalObject;
     PropertyAttributes attrs;
-    Property *p = l->lookup(o, &attrs);
-    if (p) {
+    ReturnedValue v = l->lookup(o, &attrs);
+    if (v != Primitive::emptyValue().asReturnedValue()) {
         if (attrs.isData()) {
             if (l->level == 0)
                 l->globalGetter = globalGetter0;
@@ -330,7 +592,7 @@ ReturnedValue Lookup::globalGetterGeneric(Lookup *l, ExecutionContext *ctx)
                 l->globalGetter = globalGetter1;
             else if (l->level == 2)
                 l->globalGetter = globalGetter2;
-            return p->value.asReturnedValue();
+            return v;
         } else {
             if (l->level == 0)
                 l->globalGetter = globalGetterAccessor0;
@@ -338,7 +600,7 @@ ReturnedValue Lookup::globalGetterGeneric(Lookup *l, ExecutionContext *ctx)
                 l->globalGetter = globalGetterAccessor1;
             else if (l->level == 2)
                 l->globalGetter = globalGetterAccessor2;
-            return o->getValue(p, attrs);
+            return v;
         }
     }
     Scope scope(ctx);
@@ -350,7 +612,7 @@ ReturnedValue Lookup::globalGetter0(Lookup *l, ExecutionContext *ctx)
 {
     Object *o = ctx->engine->globalObject;
     if (l->classList[0] == o->internalClass)
-        return o->memberData[l->index].value.asReturnedValue();
+        return o->memberData[l->index].asReturnedValue();
 
     l->globalGetter = globalGetterGeneric;
     return globalGetterGeneric(l, ctx);
@@ -361,7 +623,7 @@ ReturnedValue Lookup::globalGetter1(Lookup *l, ExecutionContext *ctx)
     Object *o = ctx->engine->globalObject;
     if (l->classList[0] == o->internalClass &&
         l->classList[1] == o->prototype()->internalClass)
-        return o->prototype()->memberData[l->index].value.asReturnedValue();
+        return o->prototype()->memberData[l->index].asReturnedValue();
 
     l->globalGetter = globalGetterGeneric;
     return globalGetterGeneric(l, ctx);
@@ -375,7 +637,7 @@ ReturnedValue Lookup::globalGetter2(Lookup *l, ExecutionContext *ctx)
         if (l->classList[1] == o->internalClass) {
             o = o->prototype();
             if (l->classList[2] == o->internalClass) {
-                return o->prototype()->memberData[l->index].value.asReturnedValue();
+                return o->prototype()->memberData[l->index].asReturnedValue();
             }
         }
     }
@@ -388,7 +650,7 @@ ReturnedValue Lookup::globalGetterAccessor0(Lookup *l, ExecutionContext *ctx)
     Object *o = ctx->engine->globalObject;
     if (l->classList[0] == o->internalClass) {
         Scope scope(o->engine());
-        FunctionObject *getter = o->memberData[l->index].getter();
+        FunctionObject *getter = o->propertyAt(l->index)->getter();
         if (!getter)
             return Encode::undefined();
 
@@ -406,7 +668,7 @@ ReturnedValue Lookup::globalGetterAccessor1(Lookup *l, ExecutionContext *ctx)
     if (l->classList[0] == o->internalClass &&
         l->classList[1] == o->prototype()->internalClass) {
         Scope scope(o->engine());
-        FunctionObject *getter = o->prototype()->memberData[l->index].getter();
+        FunctionObject *getter = o->prototype()->propertyAt(l->index)->getter();
         if (!getter)
             return Encode::undefined();
 
@@ -427,7 +689,7 @@ ReturnedValue Lookup::globalGetterAccessor2(Lookup *l, ExecutionContext *ctx)
             o = o->prototype();
             if (l->classList[2] == o->internalClass) {
                 Scope scope(o->engine());
-                FunctionObject *getter = o->memberData[l->index].getter();
+                FunctionObject *getter = o->propertyAt(l->index)->getter();
                 if (!getter)
                     return Encode::undefined();
 
@@ -446,7 +708,7 @@ void Lookup::setterGeneric(Lookup *l, const ValueRef object, const ValueRef valu
     Scope scope(l->name->engine());
     ScopedObject o(scope, object);
     if (!o) {
-        o = __qmljs_convert_to_object(scope.engine->currentContext(), object);
+        o = RuntimeHelpers::convertToObject(scope.engine->currentContext(), object);
         if (!o) // type error
             return;
         ScopedString s(scope, l->name);
@@ -456,72 +718,119 @@ void Lookup::setterGeneric(Lookup *l, const ValueRef object, const ValueRef valu
     o->setLookup(l, value);
 }
 
+void Lookup::setterTwoClasses(Lookup *l, const ValueRef object, const ValueRef value)
+{
+    Lookup l1 = *l;
+
+    if (Object *o = object->asObject()) {
+        o->setLookup(l, value);
+
+        if (l->setter == Lookup::setter0) {
+            l->setter = setter0setter0;
+            l->classList[1] = l1.classList[0];
+            l->index2 = l1.index;
+            return;
+        }
+    }
+
+    l->setter = setterFallback;
+    setterFallback(l, object, value);
+}
+
+void Lookup::setterFallback(Lookup *l, const ValueRef object, const ValueRef value)
+{
+    QV4::Scope scope(l->name->engine());
+    QV4::ScopedObject o(scope, object->toObject(scope.engine->currentContext()));
+    if (o) {
+        QV4::ScopedString s(scope, l->name);
+        o->put(s, value);
+    }
+}
+
 void Lookup::setter0(Lookup *l, const ValueRef object, const ValueRef value)
 {
-    Object *o = object->asObject();
+    Object *o = static_cast<Object *>(object->asManaged());
     if (o && o->internalClass == l->classList[0]) {
-        o->memberData[l->index].value = *value;
+        o->memberData[l->index] = *value;
         return;
     }
 
-    l->setter = setterGeneric;
-    setterGeneric(l, object, value);
+    setterTwoClasses(l, object, value);
 }
 
 void Lookup::setterInsert0(Lookup *l, const ValueRef object, const ValueRef value)
 {
-    Object *o = object->asObject();
+    Object *o = static_cast<Object *>(object->asManaged());
     if (o && o->internalClass == l->classList[0]) {
         if (!o->prototype()) {
-            if (l->index >= o->memberDataAlloc)
+            if (l->index >= o->memberData.size())
                 o->ensureMemberIndex(l->index);
-            o->memberData[l->index].value = *value;
+            o->memberData[l->index] = *value;
             o->internalClass = l->classList[3];
             return;
         }
     }
 
-    l->setter = setterGeneric;
-    setterGeneric(l, object, value);
+    l->setter = setterFallback;
+    setterFallback(l, object, value);
 }
 
 void Lookup::setterInsert1(Lookup *l, const ValueRef object, const ValueRef value)
 {
-    Object *o = object->asObject();
+    Object *o = static_cast<Object *>(object->asManaged());
     if (o && o->internalClass == l->classList[0]) {
         Object *p = o->prototype();
         if (p && p->internalClass == l->classList[1]) {
-            if (l->index >= o->memberDataAlloc)
+            if (l->index >= o->memberData.size())
                 o->ensureMemberIndex(l->index);
-            o->memberData[l->index].value = *value;
+            o->memberData[l->index] = *value;
             o->internalClass = l->classList[3];
             return;
         }
     }
 
-    l->setter = setterGeneric;
-    setterGeneric(l, object, value);
+    l->setter = setterFallback;
+    setterFallback(l, object, value);
 }
 
 void Lookup::setterInsert2(Lookup *l, const ValueRef object, const ValueRef value)
 {
-    Object *o = object->asObject();
+    Object *o = static_cast<Object *>(object->asManaged());
     if (o && o->internalClass == l->classList[0]) {
         Object *p = o->prototype();
         if (p && p->internalClass == l->classList[1]) {
             p = p->prototype();
             if (p && p->internalClass == l->classList[2]) {
-                if (l->index >= o->memberDataAlloc)
+                if (l->index >= o->memberData.size())
                     o->ensureMemberIndex(l->index);
-                o->memberData[l->index].value = *value;
+                o->memberData[l->index] = *value;
                 o->internalClass = l->classList[3];
                 return;
             }
         }
     }
 
-    l->setter = setterGeneric;
-    setterGeneric(l, object, value);
+    l->setter = setterFallback;
+    setterFallback(l, object, value);
+}
+
+void Lookup::setter0setter0(Lookup *l, const ValueRef object, const ValueRef value)
+{
+    Object *o = static_cast<Object *>(object->asManaged());
+    if (o) {
+        if (o->internalClass == l->classList[0]) {
+            o->memberData[l->index] = *value;
+            return;
+        }
+        if (o->internalClass == l->classList[1]) {
+            o->memberData[l->index2] = *value;
+            return;
+        }
+    }
+
+    l->setter = setterFallback;
+    setterFallback(l, object, value);
+
 }
 
 QT_END_NAMESPACE
