@@ -46,7 +46,7 @@
 #include <private/qqmlcontext_p.h>
 
 #include <private/qv4engine_p.h>
-#include <private/qv4value_p.h>
+#include <private/qv4value_inl_p.h>
 #include <private/qv4objectproto_p.h>
 #include <private/qv4mm_p.h>
 #include <private/qv4function_p.h>
@@ -59,14 +59,14 @@ QT_BEGIN_NAMESPACE
 
 using namespace QV4;
 
-DEFINE_MANAGED_VTABLE(QmlContextWrapper);
+DEFINE_OBJECT_VTABLE(QmlContextWrapper);
 
 QmlContextWrapper::QmlContextWrapper(QV8Engine *engine, QQmlContextData *context, QObject *scopeObject, bool ownsContext)
     : Object(QV8Engine::getV4(engine)),
-      v8(engine), readOnly(true), ownsContext(ownsContext), isNullWrapper(false),
+      readOnly(true), ownsContext(ownsContext), isNullWrapper(false),
       context(context), scopeObject(scopeObject), idObjectsWrapper(0)
 {
-    setVTable(&static_vtbl);
+    setVTable(staticVTable());
 }
 
 QmlContextWrapper::~QmlContextWrapper()
@@ -94,7 +94,7 @@ ReturnedValue QmlContextWrapper::urlScope(QV8Engine *v8, const QUrl &url)
     context->isInternal = true;
     context->isJSContext = true;
 
-    Scoped<QmlContextWrapper> w(scope, new (v4->memoryManager) QmlContextWrapper(v8, context, 0));
+    Scoped<QmlContextWrapper> w(scope, new (v4->memoryManager) QmlContextWrapper(v8, context, 0, true));
     w->isNullWrapper = true;
     return w.asReturnedValue();
 }
@@ -181,7 +181,7 @@ ReturnedValue QmlContextWrapper::get(Managed *m, const StringRef name, bool *has
     //     context = context->parent
     // }
 
-    QV8Engine *engine = resource->v8;
+    QV8Engine *engine = v4->v8Engine;
 
     QObject *scopeObject = resource->getScopeObject();
 
@@ -288,6 +288,13 @@ void QmlContextWrapper::put(Managed *m, const StringRef name, const ValueRef val
         return;
     }
 
+    PropertyAttributes attrs;
+    Property *pd  = wrapper->__getOwnProperty__(name, &attrs);
+    if (pd) {
+        wrapper->putValue(pd, attrs, value);
+        return;
+    }
+
     if (wrapper->isNullWrapper) {
         if (wrapper && wrapper->readOnly) {
             QString error = QLatin1String("Invalid write to global property \"") + name->toQString() +
@@ -301,13 +308,6 @@ void QmlContextWrapper::put(Managed *m, const StringRef name, const ValueRef val
         return;
     }
 
-    PropertyAttributes attrs;
-    Property *pd  = wrapper->__getOwnProperty__(name, &attrs);
-    if (pd) {
-        wrapper->putValue(pd, attrs, value);
-        return;
-    }
-
     // Its possible we could delay the calculation of the "actual" context (in the case
     // of sub contexts) until it is definately needed.
     QQmlContextData *context = wrapper->getContext();
@@ -382,9 +382,12 @@ void QmlContextWrapper::registerQmlDependencies(ExecutionEngine *engine, const C
 
     const quint32 *idObjectDependency = compiledFunction->qmlIdObjectDependencyTable();
     const int idObjectDependencyCount = compiledFunction->nDependingIdObjects;
-    for (int i = 0; i < idObjectDependencyCount; ++i, ++idObjectDependency)
+    for (int i = 0; i < idObjectDependencyCount; ++i, ++idObjectDependency) {
+        Q_ASSERT(int(*idObjectDependency) < qmlContext->idValueCount);
         capture->captureProperty(&qmlContext->idValues[*idObjectDependency].bindings);
+    }
 
+    Q_ASSERT(qmlContext->contextObject);
     const quint32 *contextPropertyDependency = compiledFunction->qmlContextPropertiesDependencyTable();
     const int contextPropertyDependencyCount = compiledFunction->nDependingContextProperties;
     for (int i = 0; i < contextPropertyDependencyCount; ++i) {
@@ -413,7 +416,7 @@ ReturnedValue QmlContextWrapper::idObjectsArray()
     return idObjectsWrapper->asReturnedValue();
 }
 
-ReturnedValue QmlContextWrapper::qmlSingletonWrapper(const StringRef &name)
+ReturnedValue QmlContextWrapper::qmlSingletonWrapper(QV8Engine *v8, const StringRef &name)
 {
     if (!context->imports)
         return Encode::undefined();
@@ -423,6 +426,7 @@ ReturnedValue QmlContextWrapper::qmlSingletonWrapper(const StringRef &name)
     Q_ASSERT(r.isValid());
     Q_ASSERT(r.type);
     Q_ASSERT(r.type->isSingleton());
+    Q_ASSERT(v8);
 
     QQmlEngine *e = v8->engine();
     QQmlType::SingletonInstanceInfo *siinfo = r.type->singletonInstanceInfo();
@@ -433,13 +437,13 @@ ReturnedValue QmlContextWrapper::qmlSingletonWrapper(const StringRef &name)
     return QJSValuePrivate::get(siinfo->scriptApi(e))->getValue(engine());
 }
 
-DEFINE_MANAGED_VTABLE(QQmlIdObjectsArray);
+DEFINE_OBJECT_VTABLE(QQmlIdObjectsArray);
 
 QQmlIdObjectsArray::QQmlIdObjectsArray(ExecutionEngine *engine, QmlContextWrapper *contextWrapper)
     : Object(engine)
     , contextWrapper(contextWrapper)
 {
-    setVTable(&static_vtbl);
+    setVTable(staticVTable());
 }
 
 ReturnedValue QQmlIdObjectsArray::getIndexed(Managed *m, uint index, bool *hasProperty)
@@ -457,6 +461,9 @@ ReturnedValue QQmlIdObjectsArray::getIndexed(Managed *m, uint index, bool *hasPr
         return Encode::undefined();
     }
 
+    if (hasProperty)
+        *hasProperty = true;
+
     ExecutionEngine *v4 = m->engine();
     QQmlEnginePrivate *ep = v4->v8Engine->engine() ? QQmlEnginePrivate::get(v4->v8Engine->engine()) : 0;
     if (ep)
