@@ -72,18 +72,23 @@ class QHashedString;
 class QHashedStringRef;
 class QReadWriteLock;
 
+namespace QV4 { struct String; }
+
 class Q_QML_PRIVATE_EXPORT QQmlMetaType
 {
 public:
     static QList<QString> qmlTypeNames();
     static QList<QQmlType*> qmlTypes();
     static QList<QQmlType*> qmlSingletonTypes();
+    static QList<QQmlType*> qmlAllTypes();
 
     static QQmlType *qmlType(const QString &qualifiedName, int, int);
     static QQmlType *qmlType(const QHashedStringRef &name, const QHashedStringRef &module, int, int);
     static QQmlType *qmlType(const QMetaObject *);
     static QQmlType *qmlType(const QMetaObject *metaObject, const QHashedStringRef &module, int version_major, int version_minor);
     static QQmlType *qmlType(int);
+    static QQmlType *qmlType(const QUrl &url, bool includeNonFileImports = false);
+    static QQmlType *qmlTypeFromIndex(int);
 
     static QMetaProperty defaultProperty(const QMetaObject *);
     static QMetaProperty defaultProperty(QObject *);
@@ -99,7 +104,7 @@ public:
 
     enum TypeCategory { Unknown, Object, List };
     static TypeCategory typeCategory(int);
-        
+
     static bool isInterface(int);
     static const char *interfaceIId(int);
     static bool isList(int);
@@ -109,15 +114,13 @@ public:
     static StringConverter customStringConverter(int);
 
     static bool isAnyModule(const QString &uri);
+    static bool isLockedModule(const QString &uri, int majorVersion);
     static bool isModule(const QString &module, int versionMajor, int versionMinor);
     static QQmlTypeModule *typeModule(const QString &uri, int majorVersion);
 
     static QList<QQmlPrivate::AutoParentFunction> parentFunctions();
 
-    static int QQuickAnchorLineMetaTypeId();
-    typedef bool (*CompareFunction)(const void *, const void *);
-    static void setQQuickAnchorLineCompareFunction(CompareFunction);
-    static bool QQuickAnchorLineCompare(const void *p1, const void *p2);
+    static const QQmlPrivate::CachedQmlUnit *findCachedCompilationUnit(const QUrl &uri);
 
     static bool namespaceContainsRegistrations(const QString &);
 
@@ -127,14 +130,10 @@ public:
     static QStringList typeRegistrationFailures();
 
     static QReadWriteLock *typeRegistrationLock();
-
-private:
-    static CompareFunction anchorLineCompareFunction;
 };
 
 struct QQmlMetaTypeData;
 class QHashedCStringRef;
-class QHashedV8String;
 class Q_QML_PRIVATE_EXPORT QQmlType
 {
 public:
@@ -164,6 +163,9 @@ public:
 
     bool isSingleton() const;
     bool isInterface() const;
+    bool isComposite() const;
+    bool isCompositeSingleton() const;
+
     int typeId() const;
     int qListTypeId() const;
 
@@ -193,6 +195,7 @@ public:
         QObject *(*qobjectCallback)(QQmlEngine *, QJSEngine *);
         const QMetaObject *instanceMetaObject;
         QString typeName;
+        QUrl url; // used by composite singletons
 
         void setQObjectApi(QQmlEngine *, QObject *);
         QObject *qobjectApi(QQmlEngine *) const;
@@ -207,9 +210,11 @@ public:
     };
     SingletonInstanceInfo *singletonInstanceInfo() const;
 
+    QUrl sourceUrl() const;
+
     int enumValue(const QHashedStringRef &, bool *ok) const;
     int enumValue(const QHashedCStringRef &, bool *ok) const;
-    int enumValue(const QHashedV8String &, bool *ok) const;
+    int enumValue(const QV4::String *, bool *ok) const;
 private:
     QQmlType *superType() const;
     friend class QQmlTypePrivate;
@@ -217,17 +222,25 @@ private:
 
     enum RegistrationType {
         CppType = 0,
-        SingletonType = 1
-        // In the future, we should register all types via QQmlType, including Composite types.
+        SingletonType = 1,
+        InterfaceType = 2,
+        CompositeType = 3,
+        CompositeSingletonType = 4
     };
     friend QString registrationTypeString(RegistrationType);
-    friend bool checkRegistration(RegistrationType, QQmlMetaTypeData *, const char *, const QString &);
+    friend bool checkRegistration(RegistrationType, QQmlMetaTypeData *, const char *, const QString &, int);
     friend int registerType(const QQmlPrivate::RegisterType &);
     friend int registerSingletonType(const QQmlPrivate::RegisterSingletonType &);
     friend int registerInterface(const QQmlPrivate::RegisterInterface &);
+    friend int registerCompositeType(const QQmlPrivate::RegisterCompositeType &);
+    friend int registerCompositeSingletonType(const QQmlPrivate::RegisterCompositeSingletonType &);
+    friend int registerQmlUnitCacheHook(const QQmlPrivate::RegisterQmlUnitCacheHook &);
+    friend Q_QML_EXPORT void qmlClearTypeRegistrations();
     QQmlType(int, const QQmlPrivate::RegisterInterface &);
     QQmlType(int, const QString &, const QQmlPrivate::RegisterSingletonType &);
     QQmlType(int, const QString &, const QQmlPrivate::RegisterType &);
+    QQmlType(int, const QString &, const QQmlPrivate::RegisterCompositeType &);
+    QQmlType(int, const QString &, const QQmlPrivate::RegisterCompositeSingletonType &);
     ~QQmlType();
 
     QQmlTypePrivate *d;
@@ -244,21 +257,23 @@ public:
     int maximumMinorVersion() const;
 
     QQmlType *type(const QHashedStringRef &, int);
-    QQmlType *type(const QHashedV8String &, int);
+    QQmlType *type(const QV4::String *, int);
 
     QList<QQmlType*> singletonTypes(int) const;
 
 private:
-    friend int registerType(const QQmlPrivate::RegisterType &);
-    friend int registerSingletonType(const QQmlPrivate::RegisterSingletonType &);
+    //Used by register functions and creates the QQmlTypeModule for them
+    friend void addTypeToData(QQmlType* type, QQmlMetaTypeData *data);
     friend struct QQmlMetaTypeData;
+    friend Q_QML_EXPORT void qmlClearTypeRegistrations();
+    friend class QQmlTypeModulePrivate;
 
     QQmlTypeModule();
     ~QQmlTypeModule();
     QQmlTypeModulePrivate *d;
 };
 
-class QQmlTypeModuleVersion 
+class QQmlTypeModuleVersion
 {
 public:
     QQmlTypeModuleVersion();
@@ -270,7 +285,7 @@ public:
     int minorVersion() const;
 
     QQmlType *type(const QHashedStringRef &) const;
-    QQmlType *type(const QHashedV8String &) const;
+    QQmlType *type(const QV4::String *) const;
 
 private:
     QQmlTypeModule *m_module;
