@@ -47,15 +47,14 @@
 #include <QtCore/qshareddata.h>
 #include <QtCore/qmetatype.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
 class QObject;
 class QQmlContext;
 class QQmlScriptStringPrivate;
-class Q_QML_EXPORT QQmlScriptString 
+class QQmlObjectCreator;
+class Q_QML_EXPORT QQmlScriptString
 {
 public:
     QQmlScriptString();
@@ -76,8 +75,8 @@ private:
     QQmlScriptString(const QString &script, QQmlContext *context, QObject *scope);
     QSharedDataPointer<QQmlScriptStringPrivate> d;
 
+    friend class QQmlObjectCreator;
     friend class QQmlScriptStringPrivate;
-    friend class QQmlVME;
     friend class QQmlExpression;
     friend class QQmlBinding;
 };
@@ -86,7 +85,5 @@ QT_END_NAMESPACE
 
 Q_DECLARE_METATYPE(QQmlScriptString)
 
-QT_END_HEADER
-
 #endif // QQMLSCRIPTSTRING_H
 
