@@ -48,7 +48,6 @@
 #include <private/qqmllocale_p.h>
 #include <private/qv8engine_p.h>
 
-#include <private/qv4profilerservice_p.h>
 #include <private/qqmlprofilerservice_p.h>
 #include <private/qqmlglobal_p.h>
 
@@ -981,12 +980,13 @@ ReturnedValue QtObject::method_createQmlObject(CallContext *ctx)
     QQmlEngine *engine = v8engine->engine();
 
     QQmlContextData *context = v8engine->callingContext();
+    Q_ASSERT(context);
     QQmlContext *effectiveContext = 0;
     if (context->isPragmaLibraryContext)
         effectiveContext = engine->rootContext();
     else
         effectiveContext = context->asQQmlContext();
-    Q_ASSERT(context && effectiveContext);
+    Q_ASSERT(effectiveContext);
 
     QString qml = ctx->callData->args[0].toQStringNoThrow();
     if (qml.isEmpty())
@@ -1087,10 +1087,10 @@ ReturnedValue QtObject::method_createComponent(CallContext *ctx)
     QQmlEngine *engine = v8engine->engine();
 
     QQmlContextData *context = v8engine->callingContext();
+    Q_ASSERT(context);
     QQmlContextData *effectiveContext = context;
     if (context->isPragmaLibraryContext)
         effectiveContext = 0;
-    Q_ASSERT(context);
 
     QString arg = ctx->callData->args[0].toQStringNoThrow();
     if (arg.isEmpty())
@@ -1208,18 +1208,16 @@ DEFINE_OBJECT_VTABLE(QQmlBindingFunction);
 /*!
     \qmlmethod Qt::binding(function)
 
-    Returns a JS object representing a binding expression which may be
-    assigned to any property in imperative code to cause a binding
-    assignment.
+    Returns a JavaScript object representing a \l{Property Binding}{property binding}.
 
-    There are two main use-cases for the function: firstly, in imperative
-    JavaScript code to cause a binding assignment:
+    There are two main use-cases for the function: firstly, to apply a
+    property binding imperatively from JavaScript code:
 
     \snippet qml/qtBinding.1.qml 0
 
-    and secondly, when defining initial property values of dynamically
-    constructed objects (via Component.createObject() or
-    Loader.setSource()) as being bound to the result of an expression.
+    and secondly, to apply a property binding when initializing property values
+    of dynamically constructed objects (via \l{Component::createObject()}
+    {Component.createObject()} or \l{Loader::setSource()}{Loader.setSource()}).
 
     For example, assuming the existence of a DynamicText component:
     \snippet qml/DynamicText.qml 0
@@ -1245,8 +1243,8 @@ DEFINE_OBJECT_VTABLE(QQmlBindingFunction);
 
     \snippet qml/qtBinding.4.qml 0
 
-    \note In \l {Qt Quick 1}, all function assignment was treated as
-    binding assignment, so the Qt.binding() function is new in
+    \note In \l {Qt Quick 1}, all function assignments were treated as
+    binding assignments. The Qt.binding() function is new to
     \l {Qt Quick}{Qt Quick 2}.
 
     \since 5.0
