@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -55,21 +55,20 @@
 
 #include <QtCore/QScopedPointer>
 
-#include <private/qquickwindowmanager_p.h>
+#include <private/qsgrenderloop_p.h>
 #include <private/qtquickglobal_p.h>
 #include <QtQuick/private/qsgcontext_p.h>
 
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QQuickWindow;
 class QSGContext;
+class QSGRenderContext;
 class QAnimationDriver;
 class QOpenGLContext;
 
-class DesignerWindowManager : public QObject, public QQuickWindowManager
+class DesignerWindowManager : public QSGRenderLoop
 {
     Q_OBJECT
 public:
@@ -83,25 +82,25 @@ public:
     void makeOpenGLContext(QQuickWindow *window);
     void exposureChanged(QQuickWindow *window);
     QImage grab(QQuickWindow *window);
-    void resize(QQuickWindow *window, const QSize &size);
 
     void maybeUpdate(QQuickWindow *window);
     void update(QQuickWindow *window); // identical for this implementation.
 
-    void releaseResources() { }
+    void releaseResources(QQuickWindow *) { }
 
     QAnimationDriver *animationDriver() const { return 0; }
 
     QSGContext *sceneGraphContext() const;
+    QSGRenderContext *createRenderContext(QSGContext *) const { return m_renderContext.data(); }
 
     static void createOpenGLContext(QQuickWindow *window);
 
 private:
     QScopedPointer<QOpenGLContext> m_openGlContext;
     QScopedPointer<QSGContext> m_sgContext;
+    QScopedPointer<QSGRenderContext> m_renderContext;
 };
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
 #endif // DESIGNERWINDOWMANAGER_P_H
