@@ -90,6 +90,8 @@ struct GLAcquireContext {
 
 QQuickContext2DTexture::QQuickContext2DTexture()
     : m_context(0)
+    , m_gl(0)
+    , m_surface(0)
     , m_item(0)
     , m_canvasWindowChanged(false)
     , m_dirtyTexture(false)
@@ -146,8 +148,12 @@ void QQuickContext2DTexture::setAntialiasing(bool antialiasing)
 void QQuickContext2DTexture::setItem(QQuickCanvasItem* item)
 {
     m_item = item;
-    m_context = (QQuickContext2D*)item->rawContext(); // FIXME
-    m_state = m_context->state;
+    if (m_item) {
+        m_context = (QQuickContext2D*) item->rawContext(); // FIXME
+        m_state = m_context->state;
+    } else {
+        m_context = 0;
+    }
 }
 
 bool QQuickContext2DTexture::setCanvasWindow(const QRect& r)
@@ -239,12 +245,15 @@ bool QQuickContext2DTexture::canvasDestroyed()
 
 void QQuickContext2DTexture::paint(QQuickContext2DCommandBuffer *ccb)
 {
+    QQuickContext2D::mutex.lock();
     if (canvasDestroyed()) {
         delete ccb;
+        QQuickContext2D::mutex.unlock();
         return;
     }
+    QQuickContext2D::mutex.unlock();
 
-    GLAcquireContext currentContext(m_context->glContext(), m_context->surface());
+    GLAcquireContext currentContext(m_gl, m_surface);
 
     if (!m_tiledCanvas) {
         paintWithoutTiles(ccb);
@@ -414,7 +423,8 @@ QQuickContext2DFBOTexture::~QQuickContext2DFBOTexture()
     delete m_multisampledFbo;
     delete m_paint_device;
 
-    glDeleteTextures(2, m_displayTextures);
+    if (QOpenGLContext::currentContext())
+        glDeleteTextures(2, m_displayTextures);
 }
 
 QSGTexture *QQuickContext2DFBOTexture::textureForNextFrame(QSGTexture *lastTexture)
@@ -433,7 +443,7 @@ QSGTexture *QQuickContext2DFBOTexture::textureForNextFrame(QSGTexture *lastTextu
         }
 
         if (m_dirtyTexture) {
-            if (!m_context->glContext()) {
+            if (!m_gl) {
                 // on a rendering thread, use the fbo directly...
                 texture->setTextureId(m_fbo->texture());
             } else {
@@ -479,7 +489,7 @@ bool QQuickContext2DFBOTexture::doMultisampling() const
     static bool multisamplingSupported = false;
 
     if (!extensionsChecked) {
-        const QSet<QByteArray> extensions = m_context->glContext()->extensions();
+        const QSet<QByteArray> extensions = QOpenGLContext::currentContext()->extensions();
         multisamplingSupported = extensions.contains(QByteArrayLiteral("GL_EXT_framebuffer_multisample"))
             && extensions.contains(QByteArrayLiteral("GL_EXT_framebuffer_blit"));
         extensionsChecked = true;
@@ -491,18 +501,18 @@ bool QQuickContext2DFBOTexture::doMultisampling() const
 void QQuickContext2DFBOTexture::grabImage(const QRectF& rf)
 {
     Q_ASSERT(rf.isValid());
-    if (!m_fbo) {
-        m_context->setGrabbedImage(QImage());
-        return;
+    QQuickContext2D::mutex.lock();
+    if (m_context) {
+        if (!m_fbo) {
+            m_context->setGrabbedImage(QImage());
+        } else {
+            QImage grabbed;
+            GLAcquireContext ctx(m_gl, m_surface);
+            grabbed = m_fbo->toImage().mirrored().copy(rf.toRect());
+            m_context->setGrabbedImage(grabbed);
+        }
     }
-
-    QImage grabbed;
-    {
-        GLAcquireContext ctx(m_context->glContext(), m_context->surface());
-        grabbed = m_fbo->toImage().mirrored().copy(rf.toRect());
-    }
-
-    m_context->setGrabbedImage(grabbed);
+    QQuickContext2D::mutex.unlock();
 }
 
 void QQuickContext2DFBOTexture::compositeTile(QQuickContext2DTile* tile)
@@ -585,7 +595,7 @@ void QQuickContext2DFBOTexture::endPainting()
     if (m_multisampledFbo)
         QOpenGLFramebufferObject::blitFramebuffer(m_fbo, m_multisampledFbo);
 
-    if (m_context->glContext()) {
+    if (m_gl) {
         /* When rendering happens on the render thread, the fbo's texture is
          * used directly for display. If we are on the GUI thread or a
          * dedicated Canvas render thread, we need to decouple the FBO from
@@ -640,9 +650,12 @@ QQuickContext2DTile* QQuickContext2DImageTexture::createTile() const
 void QQuickContext2DImageTexture::grabImage(const QRectF& rf)
 {
     Q_ASSERT(rf.isValid());
-    Q_ASSERT(m_context);
-    QImage grabbed = m_displayImage.copy(rf.toRect());
-    m_context->setGrabbedImage(grabbed);
+    QQuickContext2D::mutex.lock();
+    if (m_context) {
+        QImage grabbed = m_displayImage.copy(rf.toRect());
+        m_context->setGrabbedImage(grabbed);
+    }
+    QQuickContext2D::mutex.unlock();
 }
 
 QSGTexture *QQuickContext2DImageTexture::textureForNextFrame(QSGTexture *last)
