@@ -138,7 +138,7 @@ QT_BEGIN_NAMESPACE
     to use \l{BorderImage::horizontalTileMode}{BorderImage.Round} instead of
     \l{BorderImage::horizontalTileMode}{BorderImage.Repeat} in cases like these.
 
-    The \l{declarative/imageelements/borderimage}{BorderImage example} shows how a BorderImage
+    The Border Image example in \l{Qt Quick Examples - Image Elements} shows how a BorderImage
     can be used to simulate a shadow effect on a rectangular item.
 
     \section1 Image Loading
@@ -619,6 +619,7 @@ QSGNode *QQuickBorderImage::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeDat
     node->setSubSourceRect(QRectF(0, 0, hTiles, vTiles));
     node->setMirror(d->mirror);
 
+    node->setMipmapFiltering(QSGTexture::None);
     node->setFiltering(d->smooth ? QSGTexture::Linear : QSGTexture::Nearest);
     if (innerSourceRect == QRectF(0, 0, 1, 1) && (vTiles > 1 || hTiles > 1)) {
         node->setHorizontalWrapMode(QSGTexture::Repeat);
