@@ -327,7 +327,7 @@ bool QQuickFlickablePrivate::flick(AxisData &data, qreal minExtent, qreal maxExt
         maxDistance = qAbs(maxExtent - data.move.value());
         data.flickTarget = maxExtent;
     }
-    if (maxDistance > 0) {
+    if (maxDistance > 0 || boundsBehavior == QQuickFlickable::DragAndOvershootBounds) {
         qreal v = velocity;
         if (maxVelocity != -1 && maxVelocity < qAbs(v)) {
             if (v < 0)
@@ -548,19 +548,23 @@ is finished.
 */
 
 /*!
-    \qmlsignal QtQuick::Flickable::onDragStarted()
+    \qmlsignal QtQuick::Flickable::dragStarted()
 
-    This handler is called when the view starts to be dragged due to user
+    This signal is emitted when the view starts to be dragged due to user
     interaction.
+
+    The corresponding handler is \c onDragStarted.
 */
 
 /*!
-    \qmlsignal QtQuick::Flickable::onDragEnded()
+    \qmlsignal QtQuick::Flickable::dragEnded()
 
-    This handler is called when the user stops dragging the view.
+    This signal is emitted when the user stops dragging the view.
 
     If the velocity of the drag is sufficient at the time the
     touch/mouse button is released then a flick will start.
+
+    The corresponding handler is \c onDragEnded.
 */
 
 /*!
@@ -615,34 +619,42 @@ is finished.
 */
 
 /*!
-    \qmlsignal QtQuick::Flickable::onMovementStarted()
+    \qmlsignal QtQuick::Flickable::movementStarted()
 
-    This handler is called when the view begins moving due to user
+    This signal is emitted when the view begins moving due to user
     interaction.
+
+    The corresponding handler is \c onMovementStarted.
 */
 
 /*!
-    \qmlsignal QtQuick::Flickable::onMovementEnded()
+    \qmlsignal QtQuick::Flickable::movementEnded()
 
-    This handler is called when the view stops moving due to user
-    interaction.  If a flick was generated, this handler will
-    be triggered once the flick stops.  If a flick was not
-    generated, the handler will be triggered when the
+    This signal is emitted when the view stops moving due to user
+    interaction.  If a flick was generated, this signal will
+    be emitted once the flick stops.  If a flick was not
+    generated, this signal will be emitted when the
     user stops dragging - i.e. a mouse or touch release.
+
+    The corresponding handler is \c onMovementEnded.
 */
 
 /*!
-    \qmlsignal QtQuick::Flickable::onFlickStarted()
+    \qmlsignal QtQuick::Flickable::flickStarted()
 
-    This handler is called when the view is flicked.  A flick
+    This signal is emitted when the view is flicked.  A flick
     starts from the point that the mouse or touch is released,
     while still in motion.
+
+    The corresponding handler is \c onFlickStarted.
 */
 
 /*!
-    \qmlsignal QtQuick::Flickable::onFlickEnded()
+    \qmlsignal QtQuick::Flickable::flickEnded()
 
-    This handler is called when the view stops moving due to a flick.
+    This signal is emitted when the view stops moving due to a flick.
+
+    The corresponding handler is \c onFlickEnded.
 */
 
 /*!
@@ -665,7 +677,7 @@ is finished.
     \dots 8
     \snippet qml/flickableScrollbar.qml 1
 
-    \sa {declarative/ui-components/scrollbar}{scrollbar example}
+    \sa {customitems/scrollbar}{UI Components: Scrollbar Example}
 */
 QQuickFlickable::QQuickFlickable(QQuickItem *parent)
   : QQuickItem(*(new QQuickFlickablePrivate), parent)
@@ -751,15 +763,8 @@ void QQuickFlickable::setInteractive(bool interactive)
     Q_D(QQuickFlickable);
     if (interactive != d->interactive) {
         d->interactive = interactive;
-        if (!interactive && (d->hData.flicking || d->vData.flicking)) {
-            d->clearTimeline();
-            d->hData.vTime = d->vData.vTime = d->timeline.time();
-            d->hData.flicking = false;
-            d->vData.flicking = false;
-            emit flickingChanged();
-            emit flickingHorizontallyChanged();
-            emit flickingVerticallyChanged();
-            emit flickEnded();
+        if (!interactive) {
+            d->cancelInteraction();
         }
         emit interactiveChanged();
     }
@@ -1015,11 +1020,11 @@ void QQuickFlickablePrivate::handleMouseMoveEvent(QMouseEvent *event)
             const qreal minY = vData.dragMinBound + vData.startMargin;
             const qreal maxY = vData.dragMaxBound - vData.endMargin;
             if (boundsBehavior == QQuickFlickable::StopAtBounds) {
-                if (newY <= maxY) {
+                if (fuzzyLessThanOrEqualTo(newY, maxY)) {
                     newY = maxY;
                     rejectY = vData.pressPos == maxY && vData.move.value() == maxY && dy < 0;
                 }
-                if (newY >= minY) {
+                if (fuzzyLessThanOrEqualTo(minY, newY)) {
                     newY = minY;
                     rejectY = vData.pressPos == minY && vData.move.value() == minY && dy > 0;
                 }
@@ -1049,11 +1054,11 @@ void QQuickFlickablePrivate::handleMouseMoveEvent(QMouseEvent *event)
             const qreal minX = hData.dragMinBound + hData.startMargin;
             const qreal maxX = hData.dragMaxBound - hData.endMargin;
             if (boundsBehavior == QQuickFlickable::StopAtBounds) {
-                if (newX <= maxX) {
+                if (fuzzyLessThanOrEqualTo(newX, maxX)) {
                     newX = maxX;
                     rejectX = hData.pressPos == maxX && hData.move.value() == maxX && dx < 0;
                 }
-                if (newX >= minX) {
+                if (fuzzyLessThanOrEqualTo(minX, newX)) {
                     newX = minX;
                     rejectX = hData.pressPos == minX && hData.move.value() == minX && dx > 0;
                 }
@@ -1234,7 +1239,7 @@ void QQuickFlickable::mouseReleaseEvent(QMouseEvent *event)
             d->replayDelayedPress();
 
             // Now send the release
-            if (window()->mouseGrabberItem()) {
+            if (window() && window()->mouseGrabberItem()) {
                 QPointF localPos = window()->mouseGrabberItem()->mapFromScene(event->windowPos());
                 QScopedPointer<QMouseEvent> mouseEvent(QQuickWindowPrivate::cloneMouseEvent(event, &localPos));
                 window()->sendEvent(window()->mouseGrabberItem(), mouseEvent.data());
@@ -1262,6 +1267,8 @@ void QQuickFlickable::wheelEvent(QWheelEvent *event)
         return;
     }
 
+    event->setAccepted(false);
+
     int yDelta = event->angleDelta().y();
     int xDelta = event->angleDelta().x();
     if (yflick() && yDelta != 0) {
@@ -1360,15 +1367,15 @@ void QQuickFlickablePrivate::replayDelayedPress()
         delayedPressTimer.stop();
 
         // If we have the grab, release before delivering the event
-        QQuickWindow *w = q->window();
-        if (w && (w->mouseGrabberItem() == q)) {
-            q->ungrabMouse();
-        }
+        if (QQuickWindow *w = q->window()) {
+            if (w->mouseGrabberItem() == q)
+                q->ungrabMouse();
 
-        // Use the event handler that will take care of finding the proper item to propagate the event
-        replayingPressEvent = true;
-        QQuickWindowPrivate::get(w)->deliverMouseEvent(mouseEvent.data());
-        replayingPressEvent = false;
+            // Use the event handler that will take care of finding the proper item to propagate the event
+            replayingPressEvent = true;
+            QQuickWindowPrivate::get(w)->deliverMouseEvent(mouseEvent.data());
+            replayingPressEvent = false;
+        }
     }
 }
 
@@ -1527,6 +1534,10 @@ void QQuickFlickable::geometryChanged(const QRectF &newGeometry,
 void QQuickFlickable::flick(qreal xVelocity, qreal yVelocity)
 {
     Q_D(QQuickFlickable);
+    d->hData.reset();
+    d->vData.reset();
+    d->hData.velocity = xVelocity;
+    d->vData.velocity = yVelocity;
     bool flickedX = d->flickX(xVelocity);
     bool flickedY = d->flickY(yVelocity);
     d->flickingStarted(flickedX, flickedY);
@@ -1723,9 +1734,9 @@ void QQuickFlickable::setRebound(QQuickTransition *transition)
     and \l {Item::childrenRect.height}{childrenRect.height} properties
     of the \l contentItem. For example, the previous snippet could be rewritten with:
 
-    \qml
+    \code
     contentWidth: contentItem.childrenRect.width; contentHeight: contentItem.childrenRect.height
-    \endqml
+    \endcode
 
     Though this assumes that the origin of the childrenRect is 0,0.
 */
@@ -1997,18 +2008,24 @@ bool QQuickFlickable::yflick() const
 void QQuickFlickable::mouseUngrabEvent()
 {
     Q_D(QQuickFlickable);
-    if (d->pressed) {
-        // if our mouse grab has been removed (probably by another Flickable),
-        // fix our state
-        d->clearDelayedPress();
-        d->pressed = false;
-        d->draggingEnding();
-        d->stealMouse = false;
-        setKeepMouseGrab(false);
-        d->fixupX();
-        d->fixupY();
-        if (!d->isViewMoving())
-            movementEnding();
+    // if our mouse grab has been removed (probably by another Flickable),
+    // fix our state
+    d->cancelInteraction();
+}
+
+void QQuickFlickablePrivate::cancelInteraction()
+{
+    Q_Q(QQuickFlickable);
+    if (pressed) {
+        clearDelayedPress();
+        pressed = false;
+        draggingEnding();
+        stealMouse = false;
+        q->setKeepMouseGrab(false);
+        fixupX();
+        fixupY();
+        if (!isViewMoving())
+            q->movementEnding();
     }
 }
 
@@ -2309,6 +2326,7 @@ void QQuickFlickable::timelineCompleted()
         return;
     }
     movementEnding();
+    d->updateBeginningEnd();
 }
 
 void QQuickFlickable::movementStarting()
