@@ -62,10 +62,17 @@ public:
     {
     }
 
+    void updateTexture(QSGTexture *texture) {
+        if (m_texture == texture)
+            return;
+        m_texture = texture;
+        emit textureChanged();
+    }
+
     QSGTexture *texture() const {
         if (m_texture) {
             m_texture->setFiltering(m_smooth ? QSGTexture::Linear : QSGTexture::Nearest);
-            m_texture->setMipmapFiltering(QSGTexture::Nearest);
+            m_texture->setMipmapFiltering(m_mipmap ? QSGTexture::Linear : QSGTexture::None);
             m_texture->setHorizontalWrapMode(QSGTexture::ClampToEdge);
             m_texture->setVerticalWrapMode(QSGTexture::ClampToEdge);
         }
@@ -76,6 +83,7 @@ public:
 
     QSGTexture *m_texture;
     bool m_smooth;
+    bool m_mipmap;
 };
 
 #include "qquickimage.moc"
@@ -85,6 +93,7 @@ QQuickImagePrivate::QQuickImagePrivate()
     , paintedWidth(0)
     , paintedHeight(0)
     , pixmapChanged(false)
+    , mipmap(false)
     , hAlign(QQuickImage::AlignHCenter)
     , vAlign(QQuickImage::AlignVCenter)
     , provider(0)
@@ -99,7 +108,7 @@ QQuickImagePrivate::QQuickImagePrivate()
     \inherits Item
     \brief Displays an image
 
-    The Image type display an image.
+    The Image type displays an image.
 
     The source of the image is specified as a URL using the \l source property.
     Images can be supplied in any of the standard image formats supported by Qt,
@@ -144,7 +153,7 @@ QQuickImagePrivate::QQuickImagePrivate()
     size bounded via the \l sourceSize property. This is especially important for content
     that is loaded from external sources or provided by the user.
 
-    \sa {declarative/imageelements/image}{Image example}, QQuickImageProvider
+    \sa {Qt Quick Examples - Image Elements}, QQuickImageProvider
 */
 
 QQuickImage::QQuickImage(QQuickItem *parent)
@@ -268,7 +277,7 @@ void QQuickImagePrivate::setImage(const QImage &image)
     Note that \c clip is \c false by default which means that the item might
     paint outside its bounding rectangle even if the fillMode is set to \c PreserveAspectCrop.
 
-    \sa {declarative/imageelements/image}{Image example}
+    \sa {Qt Quick Examples - Image Elements}
 */
 QQuickImage::FillMode QQuickImage::fillMode() const
 {
@@ -365,6 +374,8 @@ qreal QQuickImage::paintedHeight() const
     no visual or performance effect.
 
     By default, this property is set to true.
+
+    \sa mipmap
 */
 
 /*!
@@ -542,7 +553,8 @@ QSGTextureProvider *QQuickImage::textureProvider() const
         QQuickImagePrivate *dd = const_cast<QQuickImagePrivate *>(d);
         dd->provider = new QQuickImageTextureProvider;
         dd->provider->m_smooth = d->smooth;
-        dd->provider->m_texture = d->sceneGraphRenderContext()->textureForFactory(d->pix.textureFactory(), window());
+        dd->provider->m_mipmap = d->mipmap;
+        dd->provider->updateTexture(d->sceneGraphRenderContext()->textureForFactory(d->pix.textureFactory(), window()));
     }
 
     return d->provider;
@@ -557,7 +569,8 @@ QSGNode *QQuickImage::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeData *)
     // Copy over the current texture state into the texture provider...
     if (d->provider) {
         d->provider->m_smooth = d->smooth;
-        d->provider->m_texture = texture;
+        d->provider->m_mipmap = d->mipmap;
+        d->provider->updateTexture(texture);
     }
 
     if (!texture || width() <= 0 || height() <= 0) {
@@ -576,8 +589,8 @@ QSGNode *QQuickImage::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeData *)
     QSGTexture::WrapMode hWrap = QSGTexture::ClampToEdge;
     QSGTexture::WrapMode vWrap = QSGTexture::ClampToEdge;
 
-    qreal pixWidth = (d->fillMode == PreserveAspectFit) ? d->paintedWidth : d->pix.width();
-    qreal pixHeight = (d->fillMode == PreserveAspectFit) ? d->paintedHeight : d->pix.height();
+    qreal pixWidth = (d->fillMode == PreserveAspectFit) ? d->paintedWidth : d->pix.width() / d->devicePixelRatio;
+    qreal pixHeight = (d->fillMode == PreserveAspectFit) ? d->paintedHeight :  d->pix.height() / d->devicePixelRatio;
 
     int xOffset = 0;
     if (d->hAlign == QQuickImage::AlignHCenter)
@@ -658,21 +671,32 @@ QSGNode *QQuickImage::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeData *)
         break;
     };
 
-    QRectF nsrect(sourceRect.x() / d->pix.width(),
-                  sourceRect.y() / d->pix.height(),
-                  sourceRect.width() / d->pix.width(),
-                  sourceRect.height() / d->pix.height());
+    qreal nsWidth = (hWrap == QSGTexture::Repeat) ? d->pix.width() / d->devicePixelRatio : d->pix.width();
+    qreal nsHeight = (vWrap == QSGTexture::Repeat) ? d->pix.height() / d->devicePixelRatio : d->pix.height();
+    QRectF nsrect(sourceRect.x() / nsWidth,
+                  sourceRect.y() / nsHeight,
+                  sourceRect.width() / nsWidth,
+                  sourceRect.height() / nsHeight);
+
+    if (targetRect.isEmpty()
+        || !qIsFinite(targetRect.width()) || !qIsFinite(targetRect.height())
+        || nsrect.isEmpty()
+        || !qIsFinite(nsrect.width()) || !qIsFinite(nsrect.height())) {
+        delete node;
+        return 0;
+    }
 
     if (d->pixmapChanged) {
         // force update the texture in the node to trigger reconstruction of
         // geometry and the likes when a atlas segment has changed.
-        if (texture->isAtlasTexture() && (hWrap == QSGTexture::Repeat || vWrap == QSGTexture::Repeat))
+        if (texture->isAtlasTexture() && (hWrap == QSGTexture::Repeat || vWrap == QSGTexture::Repeat || d->mipmap))
             node->setTexture(texture->removedFromAtlas());
         else
             node->setTexture(texture);
         d->pixmapChanged = false;
     }
 
+    node->setMipmapFiltering(d->mipmap ? QSGTexture::Linear : QSGTexture::None);
     node->setHorizontalWrapMode(hWrap);
     node->setVerticalWrapMode(vWrap);
     node->setFiltering(d->smooth ? QSGTexture::Linear : QSGTexture::Nearest);
@@ -739,4 +763,38 @@ void QQuickImage::setHorizontalAlignment(HAlignment align)
     emit horizontalAlignmentChanged(align);
 }
 
+/*!
+    \qmlproperty bool QtQuick::Image::mipmap
+    \since 5.3
+
+    This property holds whether the image uses mipmap filtering when scaled or
+    transformed.
+
+    Mipmap filtering gives better visual quality when scaling down
+    compared to smooth, but it may come at a performance cost (both when
+    initializing the image and during rendering).
+
+    By default, this property is set to false.
+
+    \sa smooth
+ */
+
+bool QQuickImage::mipmap() const
+{
+    Q_D(const QQuickImage);
+    return d->mipmap;
+}
+
+void QQuickImage::setMipmap(bool use)
+{
+    Q_D(QQuickImage);
+    if (d->mipmap == use)
+        return;
+    d->mipmap = use;
+    emit mipmapChanged(d->mipmap);
+
+    d->pixmapChanged = true;
+    update();
+}
+
 QT_END_NAMESPACE
