@@ -51,21 +51,24 @@ QT_BEGIN_NAMESPACE
 #define QML_VIEW_DEFAULTCACHEBUFFER 320
 #endif
 
-FxViewItem::FxViewItem(QQuickItem *i, QQuickItemView *v, bool own)
+FxViewItem::FxViewItem(QQuickItem *i, QQuickItemView *v, bool own, QQuickItemViewAttached *attached)
     : item(i)
     , view(v)
     , transitionableItem(0)
-    , attached(0)
+    , attached(attached)
     , ownItem(own)
     , releaseAfterTransition(false)
     , trackGeom(false)
 {
+    if (attached) // can be null for default components (see createComponentItem)
+        attached->setView(view);
 }
 
 FxViewItem::~FxViewItem()
 {
     delete transitionableItem;
     if (ownItem && item) {
+        trackGeometry(false);
         item->setParentItem(0);
         item->deleteLater();
         item = 0;
@@ -361,6 +364,8 @@ void QQuickItemView::setDelegate(QQmlComponent *delegate)
     if (!d->ownModel) {
         d->model = new QQmlDelegateModel(qmlContext(this));
         d->ownModel = true;
+        if (isComponentComplete())
+            static_cast<QQmlDelegateModel *>(d->model.data())->componentComplete();
     }
     if (QQmlDelegateModel *dataModel = qobject_cast<QQmlDelegateModel*>(d->model)) {
         int oldCount = dataModel->count();
@@ -449,6 +454,11 @@ int QQuickItemView::cacheBuffer() const
 void QQuickItemView::setCacheBuffer(int b)
 {
     Q_D(QQuickItemView);
+    if (b < 0) {
+        qmlInfo(this) << "Cannot set a negative cache buffer";
+        return;
+    }
+
     if (d->buffer != b) {
         d->buffer = b;
         if (isComponentComplete()) {
@@ -459,6 +469,41 @@ void QQuickItemView::setCacheBuffer(int b)
     }
 }
 
+int QQuickItemView::displayMarginBeginning() const
+{
+    Q_D(const QQuickItemView);
+    return d->displayMarginBeginning;
+}
+
+void QQuickItemView::setDisplayMarginBeginning(int margin)
+{
+    Q_D(QQuickItemView);
+    if (d->displayMarginBeginning != margin) {
+        d->displayMarginBeginning = margin;
+        if (isComponentComplete()) {
+            d->forceLayoutPolish();
+        }
+        emit displayMarginBeginningChanged();
+    }
+}
+
+int QQuickItemView::displayMarginEnd() const
+{
+    Q_D(const QQuickItemView);
+    return d->displayMarginEnd;
+}
+
+void QQuickItemView::setDisplayMarginEnd(int margin)
+{
+    Q_D(QQuickItemView);
+    if (d->displayMarginEnd != margin) {
+        d->displayMarginEnd = margin;
+        if (isComponentComplete()) {
+            d->forceLayoutPolish();
+        }
+        emit displayMarginEndChanged();
+    }
+}
 
 Qt::LayoutDirection QQuickItemView::layoutDirection() const
 {
@@ -648,7 +693,8 @@ void QQuickItemView::setPreferredHighlightBegin(qreal start)
     d->haveHighlightRange = d->highlightRange != NoHighlightRange && d->highlightRangeStart <= d->highlightRangeEnd;
     if (isComponentComplete()) {
         d->updateViewport();
-        d->fixupPosition();
+        if (!isMoving() && !isFlicking())
+            d->fixupPosition();
     }
     emit preferredHighlightBeginChanged();
 }
@@ -662,7 +708,8 @@ void QQuickItemView::resetPreferredHighlightBegin()
     d->highlightRangeStart = 0;
     if (isComponentComplete()) {
         d->updateViewport();
-        d->fixupPosition();
+        if (!isMoving() && !isFlicking())
+            d->fixupPosition();
     }
     emit preferredHighlightBeginChanged();
 }
@@ -683,7 +730,8 @@ void QQuickItemView::setPreferredHighlightEnd(qreal end)
     d->haveHighlightRange = d->highlightRange != NoHighlightRange && d->highlightRangeStart <= d->highlightRangeEnd;
     if (isComponentComplete()) {
         d->updateViewport();
-        d->fixupPosition();
+        if (!isMoving() && !isFlicking())
+            d->fixupPosition();
     }
     emit preferredHighlightEndChanged();
 }
@@ -697,7 +745,8 @@ void QQuickItemView::resetPreferredHighlightEnd()
     d->highlightRangeEnd = 0;
     if (isComponentComplete()) {
         d->updateViewport();
-        d->fixupPosition();
+        if (!isMoving() && !isFlicking())
+            d->fixupPosition();
     }
     emit preferredHighlightEndChanged();
 }
@@ -864,12 +913,12 @@ void QQuickItemViewPrivate::positionViewAtIndex(int index, int mode)
     else
         maxExtent = isContentFlowReversed() ? q->minXExtent()-size(): -q->maxXExtent();
     if (!item) {
-        int itemPos = positionAt(idx);
+        qreal itemPos = positionAt(idx);
         changedVisibleIndex(idx);
         // save the currently visible items in case any of them end up visible again
         QList<FxViewItem *> oldVisible = visibleItems;
         visibleItems.clear();
-        setPosition(qMin(qreal(itemPos), maxExtent));
+        setPosition(qMin(itemPos, maxExtent));
         // now release the reference to all the old visible items.
         for (int i = 0; i < oldVisible.count(); ++i)
             releaseItem(oldVisible.at(i));
@@ -1380,7 +1429,7 @@ qreal QQuickItemView::originX() const
     if (d->layoutOrientation() == Qt::Horizontal
             && effectiveLayoutDirection() == Qt::RightToLeft
             && contentWidth() < width()) {
-        return d->lastPosition() - d->footerSize();
+        return -d->lastPosition() - d->footerSize();
     }
     return QQuickFlickable::originX();
 }
@@ -1391,7 +1440,7 @@ qreal QQuickItemView::originY() const
     if (d->layoutOrientation() == Qt::Vertical
             && d->verticalLayoutDirection == QQuickItemView::BottomToTop
             && contentHeight() < height()) {
-        return d->lastPosition() - d->footerSize();
+        return -d->lastPosition() - d->footerSize();
     }
     return QQuickFlickable::originY();
 }
@@ -1443,6 +1492,7 @@ void QQuickItemView::componentComplete()
 QQuickItemViewPrivate::QQuickItemViewPrivate()
     : itemCount(0)
     , buffer(QML_VIEW_DEFAULTCACHEBUFFER), bufferMode(BufferBefore | BufferAfter)
+    , displayMarginBeginning(0), displayMarginEnd(0)
     , layoutDirection(Qt::LeftToRight), verticalLayoutDirection(QQuickItemView::TopToBottom)
     , moveReason(Other)
     , visibleIndex(0)
@@ -1682,9 +1732,9 @@ void QQuickItemViewPrivate::refill()
 {
     qreal s = qMax(size(), qreal(0.));
     if (isContentFlowReversed())
-        refill(-position()-s, -position());
+        refill(-position()-displayMarginBeginning-s, -position()+displayMarginEnd);
     else
-        refill(position(), position()+s);
+        refill(position()-displayMarginBeginning, position()+displayMarginEnd+s);
 }
 
 void QQuickItemViewPrivate::refill(qreal from, qreal to)
@@ -1733,18 +1783,20 @@ void QQuickItemViewPrivate::refill(qreal from, qreal to)
         emit q->countChanged();
 }
 
-void QQuickItemViewPrivate::regenerate()
+void QQuickItemViewPrivate::regenerate(bool orientationChanged)
 {
     Q_Q(QQuickItemView);
     if (q->isComponentComplete()) {
         currentChanges.reset();
-        delete header;
-        header = 0;
-        delete footer;
-        footer = 0;
+        if (orientationChanged) {
+            delete header;
+            header = 0;
+            delete footer;
+            footer = 0;
+        }
+        clear();
         updateHeader();
         updateFooter();
-        clear();
         updateViewport();
         setPosition(contentStartOffset());
         refill();
@@ -1835,7 +1887,7 @@ void QQuickItemViewPrivate::layout()
 
         prepareVisibleItemTransitions();
 
-        QRectF viewBounds(0, position(), q->width(), q->height());
+        QRectF viewBounds(q->contentX(),  q->contentY(), q->width(), q->height());
         for (QList<FxViewItem*>::Iterator it = releasePendingTransition.begin();
              it != releasePendingTransition.end(); ) {
             FxViewItem *item = *it;
@@ -2275,7 +2327,8 @@ void QQuickItemView::initItem(int, QObject *object)
 {
     QQuickItem* item = qmlobject_cast<QQuickItem*>(object);
     if (item) {
-        item->setZ(1);
+        if (qFuzzyIsNull(item->z()))
+            item->setZ(1);
         item->setParentItem(contentItem());
         QQuickItemPrivate::get(item)->setCulled(true);
     }
@@ -2339,7 +2392,8 @@ QQuickItem *QQuickItemViewPrivate::createComponentItem(QQmlComponent *component,
         item = new QQuickItem;
     }
     if (item) {
-        item->setZ(zValue);
+        if (qFuzzyIsNull(item->z()))
+            item->setZ(zValue);
         QQml_setParent_noEvent(item, q->contentItem());
         item->setParentItem(q->contentItem());
     }
