@@ -56,7 +56,7 @@ QT_BEGIN_NAMESPACE
 class FxViewItem
 {
 public:
-    FxViewItem(QQuickItem *, QQuickItemView *, bool own);
+    FxViewItem(QQuickItem *, QQuickItemView *, bool own, QQuickItemViewAttached *attached);
     virtual ~FxViewItem();
 
     qreal itemX() const;
@@ -184,7 +184,7 @@ public:
     virtual void clear();
     virtual void updateViewport();
 
-    void regenerate();
+    void regenerate(bool orientationChanged=false);
     void layout();
     virtual void animationFinished(QAbstractAnimationJob *);
     void refill();
@@ -257,6 +257,8 @@ public:
     int itemCount;
     int buffer;
     int bufferMode;
+    int displayMarginBeginning;
+    int displayMarginEnd;
     Qt::LayoutDirection layoutDirection;
     QQuickItemView::VerticalLayoutDirection verticalLayoutDirection;
 
