@@ -104,6 +104,7 @@ QQuickPathViewPrivate::QQuickPathViewPrivate()
     , stealMouse(false), ownModel(false), interactive(true), haveHighlightRange(true)
     , autoHighlight(true), highlightUp(false), layoutScheduled(false)
     , moving(false), flicking(false), dragging(false), inRequest(false), delegateValidated(false)
+    , inRefill(false)
     , dragMargin(0), deceleration(100), maximumFlickVelocity(QML_FLICK_DEFAULTMAXVELOCITY)
     , moveOffset(this, &QQuickPathViewPrivate::setAdjustedOffset), flickDuration(0)
     , firstIndex(-1), pathItems(-1), requestedIndex(-1), cacheSize(0), requestedZ(0)
@@ -1147,50 +1148,62 @@ bool QQuickPathView::isDragging() const
 }
 
 /*!
-    \qmlsignal QtQuick::PathView::onMovementStarted()
+    \qmlsignal QtQuick::PathView::movementStarted()
 
-    This handler is called when the view begins moving due to user
+    This signal is emitted when the view begins moving due to user
     interaction.
+
+    The corresponding handler is \c onMovementStarted.
 */
 
 /*!
-    \qmlsignal QtQuick::PathView::onMovementEnded()
+    \qmlsignal QtQuick::PathView::movementEnded()
 
-    This handler is called when the view stops moving due to user
-    interaction.  If a flick was generated, this handler will
-    be triggered once the flick stops.  If a flick was not
-    generated, the handler will be triggered when the
+    This signal is emitted when the view stops moving due to user
+    interaction.  If a flick was generated, this signal will
+    be emitted once the flick stops.  If a flick was not
+    generated, this signal will be emitted when the
     user stops dragging - i.e. a mouse or touch release.
+
+    The corresponding handler is \c onMovementEnded.
 */
 
 /*!
-    \qmlsignal QtQuick::PathView::onFlickStarted()
+    \qmlsignal QtQuick::PathView::flickStarted()
 
-    This handler is called when the view is flicked.  A flick
+    This signal is emitted when the view is flicked.  A flick
     starts from the point that the mouse or touch is released,
     while still in motion.
+
+    The corresponding handler is \c onFlickStarted.
 */
 
 /*!
-    \qmlsignal QtQuick::PathView::onFlickEnded()
+    \qmlsignal QtQuick::PathView::flickEnded()
 
-    This handler is called when the view stops moving due to a flick.
+    This signal is emitted when the view stops moving due to a flick.
+
+    The corresponding handler is \c onFlickEnded.
 */
 
 /*!
-    \qmlsignal QtQuick::PathView::onDragStarted()
+    \qmlsignal QtQuick::PathView::dragStarted()
 
-    This handler is called when the view starts to be dragged due to user
+    This signal is emitted when the view starts to be dragged due to user
     interaction.
+
+    The corresponding handler is \c onDragStarted.
 */
 
 /*!
-    \qmlsignal QtQuick::PathView::onDragEnded()
+    \qmlsignal QtQuick::PathView::dragEnded()
 
-    This handler is called when the user stops dragging the view.
+    This signal is emitted when the user stops dragging the view.
 
     If the velocity of the drag is suffient at the time the
     touch/mouse button is released then a flick will start.
+
+    The corresponding handler is \c onDragEnded.
 */
 
 /*!
@@ -1230,6 +1243,8 @@ void QQuickPathView::setDelegate(QQmlComponent *delegate)
     if (!d->ownModel) {
         d->model = new QQmlDelegateModel(qmlContext(this));
         d->ownModel = true;
+        if (isComponentComplete())
+            static_cast<QQmlDelegateModel *>(d->model.data())->componentComplete();
     }
     if (QQmlDelegateModel *dataModel = qobject_cast<QQmlDelegateModel*>(d->model)) {
         int oldCount = dataModel->count();
@@ -1584,6 +1599,7 @@ void QQuickPathViewPrivate::handleMousePressEvent(QMouseEvent *event)
         return;
 
     startPoint = pointNear(event->localPos(), &startPc);
+    startPos = event->localPos();
     if (idx == items.count()) {
         qreal distance = qAbs(event->localPos().x() - startPoint.x()) + qAbs(event->localPos().y() - startPoint.y());
         if (distance > dragMargin)
@@ -1606,8 +1622,6 @@ void QQuickPathView::mouseMoveEvent(QMouseEvent *event)
     Q_D(QQuickPathView);
     if (d->interactive) {
         d->handleMouseMoveEvent(event);
-        if (d->stealMouse)
-            setKeepMouseGrab(true);
         event->accept();
     } else {
         QQuickItem::mouseMoveEvent(event);
@@ -1624,9 +1638,17 @@ void QQuickPathViewPrivate::handleMouseMoveEvent(QMouseEvent *event)
     qreal newPc;
     QPointF pathPoint = pointNear(event->localPos(), &newPc);
     if (!stealMouse) {
-        QPointF delta = pathPoint - startPoint;
-        if (qAbs(delta.x()) > qApp->styleHints()->startDragDistance() || qAbs(delta.y()) > qApp->styleHints()->startDragDistance()) {
-            stealMouse = true;
+        QPointF posDelta = event->localPos() - startPos;
+        if (QQuickWindowPrivate::dragOverThreshold(posDelta.y(), Qt::YAxis, event) || QQuickWindowPrivate::dragOverThreshold(posDelta.x(), Qt::XAxis, event)) {
+            // The touch has exceeded the threshold. If the movement along the path is close to the drag threshold
+            // then we'll assume that this gesture targets the PathView. This ensures PathView gesture grabbing
+            // is in sync with other items.
+            QPointF pathDelta = pathPoint - startPoint;
+            if (qAbs(pathDelta.x()) > qApp->styleHints()->startDragDistance() * 0.8
+                    || qAbs(pathDelta.y()) > qApp->styleHints()->startDragDistance() * 0.8) {
+                stealMouse = true;
+                q->setKeepMouseGrab(true);
+            }
         }
     } else {
         moveReason = QQuickPathViewPrivate::Mouse;
@@ -1859,11 +1881,18 @@ void QQuickPathView::refill()
 {
     Q_D(QQuickPathView);
 
+    if (d->inRefill) {
+        d->scheduleLayout();
+        return;
+    }
+
     d->layoutScheduled = false;
 
     if (!d->isValid() || !isComponentComplete())
         return;
 
+    d->inRefill = true;
+
     bool currentVisible = false;
     int count = d->pathItems == -1 ? d->modelCount : qMin(d->pathItems, d->modelCount);
 
@@ -1996,6 +2025,8 @@ void QQuickPathView::refill()
     }
     while (d->itemCache.count())
         d->releaseItem(d->itemCache.takeLast());
+
+    d->inRefill = false;
 }
 
 void QQuickPathView::modelUpdated(const QQmlChangeSet &changeSet, bool reset)
