@@ -166,8 +166,8 @@ static bool qsg_leak_check = !qgetenv("QML_LEAK_CHECK").isEmpty();
         };
     \endcode
 
-    \warning Instances of QSGMaterialShader belongs to the Scene Graph rendering
-    thread, and cannot be used from the GUI thread.
+    \note All classes with QSG prefix should be used solely on the scene graph's
+    rendering thread. See \l {Scene Graph and Rendering} for more information.
 
  */
 
@@ -257,9 +257,6 @@ const char *QSGMaterialShader::fragmentShader() const
 
     State that is global for all uses of the shader, independent of the geometry
     that is being drawn, can be setup in this function.
-
-    If reimplemented, make sure to either call the base class implementation to
-    enable the vertex attribute registers.
  */
 
 void QSGMaterialShader::activate()
@@ -271,9 +268,6 @@ void QSGMaterialShader::activate()
 /*!
     This function is called by the scene graph to indicate that geometry will
     no longer to be rendered using this shader.
-
-    If reimplemented, make sure to either call the base class implementation to
-    disable the vertex attribute registers.
  */
 
 void QSGMaterialShader::deactivate()
@@ -556,6 +550,9 @@ static void qt_print_material_count()
     \ingroup qtquick-scenegraph-materials
 
     It serves no purpose outside the QSGMaterial::type() function.
+
+    \note All classes with QSG prefix should be used solely on the scene graph's
+    rendering thread. See \l {Scene Graph and Rendering} for more information.
  */
 
 /*!
@@ -591,8 +588,8 @@ static void qt_print_material_count()
         };
     \endcode
 
-    \warning Instances of QSGMaterial belongs to the Scene Graph rendering thread,
-    and cannot be used from the GUI thread.
+    \note All classes with QSG prefix should be used solely on the scene graph's
+    rendering thread. See \l {Scene Graph and Rendering} for more information.
  */
 
 /*!
