@@ -54,7 +54,7 @@
 
 #include <qdatetime.h>
 
-#include <private/qqmlprofilerservice_p.h>
+#include <private/qquickprofiler_p.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -103,7 +103,9 @@ QSGBindableFboId::QSGBindableFboId(GLuint id)
 
 void QSGBindableFboId::bind() const
 {
-    QOpenGLContext::currentContext()->functions()->glBindFramebuffer(GL_FRAMEBUFFER, m_id);
+    QOpenGLContext *context = QOpenGLContext::currentContext();
+    context->functions()->glBindFramebuffer(GL_FRAMEBUFFER, m_id);
+    QOpenGLContextPrivate::get(context)->current_fbo = m_id;
 }
 
 /*!
@@ -227,7 +229,7 @@ void QSGRenderer::renderScene(const QSGBindable &bindable)
 
 
 #ifndef QSG_NO_RENDER_TIMING
-    bool profileFrames = qsg_render_timing || QQmlProfilerService::enabled;
+    bool profileFrames = qsg_render_timing || QQuickProfiler::enabled;
     if (profileFrames)
         frameTimer.start();
     qint64 bindTime = 0;
@@ -287,14 +289,11 @@ void QSGRenderer::renderScene(const QSGBindable &bindable)
                int(renderTime / 1000000));
     }
 
-    if (QQmlProfilerService::enabled) {
-        QQmlProfilerService::sceneGraphFrame(
-                    QQmlProfilerService::SceneGraphRendererFrame,
-                    preprocessTime,
-                    updatePassTime - preprocessTime,
-                    bindTime - updatePassTime,
-                    renderTime - bindTime);
-    }
+    Q_QUICK_SG_PROFILE1(QQuickProfiler::SceneGraphRendererFrame, (
+            preprocessTime,
+            updatePassTime - preprocessTime,
+            bindTime - updatePassTime,
+            renderTime - bindTime));
 
 #endif
 }
@@ -378,7 +377,7 @@ void QSGRenderer::preprocess()
     }
 
 #ifndef QSG_NO_RENDER_TIMING
-    bool profileFrames = qsg_render_timing || QQmlProfilerService::enabled;
+    bool profileFrames = qsg_render_timing || QQuickProfiler::enabled;
     if (profileFrames)
         preprocessTime = frameTimer.nsecsElapsed();
 #endif
