@@ -94,6 +94,7 @@ public:
     virtual void invalidate();
 
     virtual void renderNextFrame(QSGRenderer *renderer, GLuint fboId);
+    virtual void endSync();
 
     virtual QSharedPointer<QSGDepthStencilBuffer> depthStencilBufferForFbo(QOpenGLFramebufferObject *fbo);
     QSGDepthStencilBufferManager *depthStencilBufferManager();
@@ -105,6 +106,9 @@ public:
     virtual QSGTexture *createTextureNoAtlas(const QImage &image) const;
     virtual QSGRenderer *createRenderer();
 
+    virtual void compile(QSGMaterialShader *shader, QSGMaterial *material, const char *vertexCode = 0, const char *fragmentCode = 0);
+    virtual void initialize(QSGMaterialShader *shader);
+
     void registerFontengineForCleanup(QFontEngine *engine);
 
     static QSGRenderContext *from(QOpenGLContext *context);
@@ -124,6 +128,7 @@ protected:
 
     QMutex m_mutex;
     QHash<QQuickTextureFactory *, QSGTexture *> m_textures;
+    QSet<QSGTexture *> m_texturesToDelete;
     QSGAtlasTexture::Manager *m_atlasManager;
 
     QSGDepthStencilBufferManager *m_depthStencilManager;
@@ -153,19 +158,16 @@ public:
 
     virtual void renderContextInitialized(QSGRenderContext *renderContext);
     virtual void renderContextInvalidated(QSGRenderContext *renderContext);
+    virtual QSGRenderContext *createRenderContext();
 
     virtual QSGRectangleNode *createRectangleNode();
     virtual QSGImageNode *createImageNode();
-    virtual QSGGlyphNode *createGlyphNode(QSGRenderContext *rc);
-    virtual QSGGlyphNode *createNativeGlyphNode(QSGRenderContext *rc);
+    virtual QSGGlyphNode *createGlyphNode(QSGRenderContext *rc, bool preferNativeGlyphNode);
     virtual QAnimationDriver *createAnimationDriver(QObject *parent);
 
     virtual QSize minimumFBOSize() const;
     virtual QSurfaceFormat defaultSurfaceFormat() const;
 
-    static void setSharedOpenGLContext(QOpenGLContext *context);
-    static QOpenGLContext *sharedOpenGLContext();
-
     void setDistanceFieldEnabled(bool enabled);
     bool isDistanceFieldEnabled() const;
 
