@@ -42,6 +42,7 @@
 #include "qsgwindowsrenderloop_p.h"
 
 #include <QtCore/QCoreApplication>
+#include <QtCore/QLibraryInfo>
 
 #include <QtGui/QScreen>
 #include <QtGui/QGuiApplication>
@@ -51,7 +52,7 @@
 
 #include <QtQuick/QQuickWindow>
 
-#include <private/qqmlprofilerservice_p.h>
+#include <private/qquickprofiler_p.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -69,7 +70,7 @@ static QElapsedTimer qsg_debug_timer;
 #ifndef QSG_NO_RENDER_TIMING
 static bool qsg_render_timing = !qgetenv("QSG_RENDER_TIMING").isEmpty();
 static QElapsedTimer qsg_render_timer;
-#define QSG_RENDER_TIMING_SAMPLE(sampleName) qint64 sampleName = 0; if (qsg_render_timing || QQmlProfilerService::enabled) sampleName = qsg_render_timer.nsecsElapsed()
+#define QSG_RENDER_TIMING_SAMPLE(sampleName) qint64 sampleName = 0; if (qsg_render_timing || QQuickProfiler::enabled) sampleName = qsg_render_timer.nsecsElapsed()
 #else
 #define QSG_RENDER_TIMING_SAMPLE(sampleName)
 #endif
@@ -85,7 +86,7 @@ QSGWindowsRenderLoop::QSGWindowsRenderLoop()
     qsg_debug_timer.start();
 #endif
 
-    m_rc = new QSGRenderContext(m_sg);
+    m_rc = m_sg->createRenderContext();
 
     m_animationDriver = m_sg->createAnimationDriver(m_sg);
     m_animationDriver->install();
@@ -178,15 +179,19 @@ void QSGWindowsRenderLoop::show(QQuickWindow *window)
         RLDEBUG(" - creating GL context");
         m_gl = new QOpenGLContext();
         m_gl->setFormat(window->requestedFormat());
-        if (QSGContext::sharedOpenGLContext())
-            m_gl->setShareContext(QSGContext::sharedOpenGLContext());
+        if (QOpenGLContextPrivate::globalShareContext())
+            m_gl->setShareContext(QOpenGLContextPrivate::globalShareContext());
         bool created = m_gl->create();
         if (!created) {
-            qWarning("QtQuick: failed to create OpenGL context");
+            const bool isEs = m_gl->isOpenGLES();
             delete m_gl;
             m_gl = 0;
+            handleContextCreationFailure(window, isEs);
             return;
         }
+
+        QQuickWindowPrivate::get(window)->fireOpenGLContextCreated(m_gl);
+
         QSG_RENDER_TIMING_SAMPLE(time_created);
         RLDEBUG(" - making current");
         bool current = m_gl->makeCurrent(window);
@@ -202,13 +207,10 @@ void QSGWindowsRenderLoop::show(QQuickWindow *window)
                    int((time_current - time_created)/1000000),
                    int((qsg_render_timer.nsecsElapsed() - time_current)/1000000));
         }
-        if (QQmlProfilerService::enabled) {
-            QQmlProfilerService::sceneGraphFrame(
-                        QQmlProfilerService::SceneGraphWindowsRenderShow,
-                        time_created - time_start,
-                        time_current - time_created,
-                        qsg_render_timer.nsecsElapsed() - time_current);
-        }
+        Q_QUICK_SG_PROFILE1(QQuickProfiler::SceneGraphWindowsRenderShow, (
+                time_created - time_start,
+                time_current - time_created,
+                qsg_render_timer.nsecsElapsed() - time_current));
 #endif
 
     }
@@ -245,6 +247,7 @@ void QSGWindowsRenderLoop::hide(QQuickWindow *window)
 
     QQuickWindowPrivate *cd = QQuickWindowPrivate::get(window);
     m_gl->makeCurrent(window);
+    cd->fireAboutToStop();
     cd->cleanupNodesOnShutdown();
 
     // If this is the last tracked window, check for persistent SG and GL and
@@ -338,7 +341,7 @@ QImage QSGWindowsRenderLoop::grab(QQuickWindow *window)
     d->syncSceneGraph();
     d->renderSceneGraph(window->size());
 
-    QImage image = qt_gl_read_framebuffer(window->size(), false, false);
+    QImage image = qt_gl_read_framebuffer(window->size() * window->devicePixelRatio(), false, false);
     return image;
 }
 
@@ -407,11 +410,8 @@ void QSGWindowsRenderLoop::render()
             qDebug("WindowsRenderLoop: animations=%d ms",
                    int((qsg_render_timer.nsecsElapsed() - time_start)/1000000));
         }
-        if (QQmlProfilerService::Enabled) {
-            QQmlProfilerService::sceneGraphFrame(
-                        QQmlProfilerService::SceneGraphWindowsAnimations,
-                        qsg_render_timer.nsecsElapsed() - time_start);
-        }
+        Q_QUICK_SG_PROFILE1(QQuickProfiler::SceneGraphWindowsAnimations, (
+                qsg_render_timer.nsecsElapsed() - time_start));
 #endif
 
         // It is not given that animations triggered another maybeUpdate()
@@ -448,6 +448,8 @@ void QSGWindowsRenderLoop::renderWindow(QQuickWindow *window)
     d->polishItems();
     QSG_RENDER_TIMING_SAMPLE(time_polished);
 
+    emit window->afterAnimating();
+
     RLDEBUG(" - syncing");
     d->syncSceneGraph();
     QSG_RENDER_TIMING_SAMPLE(time_synced);
@@ -473,19 +475,13 @@ void QSGWindowsRenderLoop::renderWindow(QQuickWindow *window)
                    int((time_rendered - time_synced)/1000000),
                    int((time_swapped - time_rendered)/1000000));
         }
-        if (QQmlProfilerService::enabled) {
-            QQmlProfilerService::sceneGraphFrame(
-                        QQmlProfilerService::SceneGraphWindowsPolishFrame,
-                        time_polished - time_start
-                        );
 
-            QQmlProfilerService::sceneGraphFrame(
-                        QQmlProfilerService::SceneGraphRenderLoopFrame,
-                        time_synced - time_polished,
-                        time_rendered - time_synced,
-                        time_swapped - time_rendered
-                        );
-        }
+        Q_QUICK_SG_PROFILE2(QQuickProfiler::SceneGraphWindowsPolishFrame,
+                            QQuickProfiler::SceneGraphRenderLoopFrame, (
+                time_synced - time_polished,
+                time_rendered - time_synced,
+                time_swapped - time_rendered,
+                time_polished - time_start));
 #endif
 }
 
