@@ -58,8 +58,6 @@
 
 #include "qquickanimation_p_p.h"
 
-#include "private/qparallelanimationgroupjob_p.h"
-
 #include <private/qobject_p.h>
 #include <QBasicTimer>
 
@@ -107,6 +105,7 @@ public:
 protected:
     virtual void updateCurrentTime(int);
     virtual void updateState(QAbstractAnimationJob::State, QAbstractAnimationJob::State);
+    void debugAnimation(QDebug d) const;
 
 private:
     qreal easeFollow(qreal);
@@ -129,7 +128,7 @@ private:
     qreal s;  // Total s
 
     int lastTime;
-    bool useDelta;
+    bool skipUpdate;
 
     bool recalc();
     void delayedStop();
