@@ -43,7 +43,7 @@
 
 #include "qquickanimation_p_p.h"
 #include <private/qqmlproperty_p.h>
-#include "private/qparallelanimationgroupjob_p.h"
+#include "private/qcontinuinganimationgroupjob_p.h"
 
 #include <QtCore/qdebug.h>
 
@@ -93,7 +93,7 @@ public:
 
     bool useMass : 1;
     bool haveModulus : 1;
-    bool useDelta : 1;
+    bool skipUpdate : 1;
     typedef QHash<QQmlProperty, QSpringAnimation*> ActiveAnimationHash;
 
     void clearTemplate() { animationTemplate = 0; }
@@ -101,6 +101,7 @@ public:
 protected:
     virtual void updateCurrentTime(int time);
     virtual void updateState(QAbstractAnimationJob::State, QAbstractAnimationJob::State);
+    void debugAnimation(QDebug d) const;
 
 private:
     QQuickSpringAnimationPrivate *animationTemplate;
@@ -160,7 +161,7 @@ QSpringAnimation::QSpringAnimation(QQuickSpringAnimationPrivate *priv)
     , modulus(0.0)
     , useMass(false)
     , haveModulus(false)
-    , useDelta(false)
+    , skipUpdate(false)
     , animationTemplate(priv)
 {
 }
@@ -194,11 +195,10 @@ int QSpringAnimation::duration() const
 void QSpringAnimation::restart()
 {
     if (isRunning() || (stopTime != -1 && (animationTemplate->elapsed.elapsed() - stopTime) < DELAY_STOP_TIMER_INTERVAL)) {
-        useDelta = true;
+        skipUpdate = true;
         init();
-        lastTime = 0;
     } else {
-        useDelta = false;
+        skipUpdate = false;
         //init() will be triggered when group starts
     }
 }
@@ -211,16 +211,17 @@ void QSpringAnimation::init()
 
 void QSpringAnimation::updateCurrentTime(int time)
 {
+    if (skipUpdate) {
+        skipUpdate = false;
+        return;
+    }
+
     if (mode == Track) {
         stop();
         return;
     }
 
-    int elapsed = useDelta ? QQmlAnimationTimer::instance()->currentDelta() : time - lastTime;
-    if (useDelta) {
-        startTime = time - elapsed;
-        useDelta = false;
-    }
+    int elapsed = time - lastTime;
 
     if (!elapsed)
         return;
@@ -320,6 +321,15 @@ void QSpringAnimation::updateState(QAbstractAnimationJob::State newState, QAbstr
         init();
 }
 
+void QSpringAnimation::debugAnimation(QDebug d) const
+{
+    d << "SpringAnimationJob(" << hex << (void *) this << dec << ")" << "velocity:" << maxVelocity
+      << "spring:" << spring << "damping:" << damping << "epsilon:" << epsilon << "modulus:" << modulus
+      << "mass:" << mass << "target:" << target.object() << "property:" << target.name()
+      << "to:" << to << "current velocity:" << velocity;
+}
+
+
 void QQuickSpringAnimationPrivate::updateMode()
 {
     if (spring == 0. && maxVelocity == 0.)
@@ -534,7 +544,7 @@ QAbstractAnimationJob* QQuickSpringAnimation::transition(QQuickStateActions &act
     Q_D(QQuickSpringAnimation);
     Q_UNUSED(direction);
 
-    QParallelAnimationGroupJob *wrapperGroup = new QParallelAnimationGroupJob();
+    QContinuingAnimationGroupJob *wrapperGroup = new QContinuingAnimationGroupJob();
 
     QQuickStateActions dataActions = QQuickNumberAnimation::createTransitionActions(actions, modified, defaultTarget);
     if (!dataActions.isEmpty()) {
