@@ -47,6 +47,7 @@
 #include <QtQml/QQmlEngine>
 #include <QtQml/QQmlComponent>
 #include <QtQuick/private/qquickrectangle_p.h>
+#include <QtQuick/private/qquickloader_p.h>
 #include "../../shared/util.h"
 #include "../shared/visualtestutil.h"
 #include <QSignalSpy>
@@ -250,6 +251,18 @@ int TestTouchItem::mousePressNum = 0;
 int TestTouchItem::mouseMoveNum = 0;
 int TestTouchItem::mouseReleaseNum = 0;
 
+class EventFilter : public QObject
+{
+public:
+    bool eventFilter(QObject *watched, QEvent *event) {
+        Q_UNUSED(watched);
+        events.append(event->type());
+        return false;
+    }
+
+    QList<int> events;
+};
+
 class ConstantUpdateItem : public QQuickItem
 {
 Q_OBJECT
@@ -283,6 +296,8 @@ private slots:
         QWindowSystemInterface::registerTouchDevice(touchDeviceWithVelocity);
     }
 
+    void openglContextCreatedSignal();
+    void aboutToStopSignal();
 
     void constantUpdates();
     void constantUpdatesOnWindow_data();
@@ -327,20 +342,63 @@ private slots:
 
     void requestActivate();
 
+    void testWindowVisibilityOrder();
+
     void blockClosing();
 
     void crashWhenHoverItemDeleted();
 
+    void unloadSubWindow();
+
+    void qobjectEventFilter_touch();
+    void qobjectEventFilter_key();
+    void qobjectEventFilter_mouse();
+
 #ifndef QT_NO_CURSOR
     void cursor();
 #endif
 
+    void animatingSignal();
+
+    void contentItemSize();
+
 private:
     QTouchDevice *touchDevice;
     QTouchDevice *touchDeviceWithVelocity;
 };
 
-//If the item calls update inside updatePaintNode, it should schedule another update
+Q_DECLARE_METATYPE(QOpenGLContext *);
+
+void tst_qquickwindow::openglContextCreatedSignal()
+{
+    qRegisterMetaType<QOpenGLContext *>();
+
+    QQuickWindow window;
+    QSignalSpy spy(&window, SIGNAL(openglContextCreated(QOpenGLContext*)));
+
+    window.show();
+    QTest::qWaitForWindowExposed(&window);
+
+    QVERIFY(spy.size() > 0);
+
+    QVariant ctx = spy.at(0).at(0);
+    QCOMPARE(qVariantValue<QOpenGLContext *>(ctx), window.openglContext());
+}
+
+void tst_qquickwindow::aboutToStopSignal()
+{
+    QQuickWindow window;
+    window.show();
+    QTest::qWaitForWindowExposed(&window);
+
+    QSignalSpy spy(&window, SIGNAL(sceneGraphAboutToStop()));
+
+    window.hide();
+
+    QVERIFY(spy.count() > 0);
+}
+
+//If the item calls update inside updatePaintNode, it should schedule another sync pass
 void tst_qquickwindow::constantUpdates()
 {
     QQuickWindow window;
@@ -348,10 +406,12 @@ void tst_qquickwindow::constantUpdates()
     ConstantUpdateItem item(window.contentItem());
     window.show();
 
-    QSignalSpy spy(&window, SIGNAL(beforeSynchronizing()));
+    QSignalSpy beforeSpy(&window, SIGNAL(beforeSynchronizing()));
+    QSignalSpy afterSpy(&window, SIGNAL(afterSynchronizing()));
 
     QTRY_VERIFY(item.iterations > 10);
-    QTRY_VERIFY(spy.count() > 10);
+    QTRY_VERIFY(beforeSpy.count() > 10);
+    QTRY_VERIFY(afterSpy.count() > 10);
 }
 
 void tst_qquickwindow::constantUpdatesOnWindow_data()
@@ -923,6 +983,9 @@ void tst_qquickwindow::mouseFiltering()
     QTRY_COMPARE(middleItem->mousePressId, 1);
     QTRY_COMPARE(bottomItem->mousePressId, 2);
     QTRY_COMPARE(topItem->mousePressId, 3);
+
+    // clean up mouse press state for the next tests
+    QTest::mouseRelease(window, Qt::LeftButton, 0, pos);
 }
 
 void tst_qquickwindow::qmlCreation()
@@ -1054,6 +1117,43 @@ void tst_qquickwindow::animationsWhileHidden()
     QTRY_VERIFY(window->isVisible());
 }
 
+// When running on native Nvidia graphics cards on linux, the
+// distance field glyph pixels have a measurable, but not visible
+// pixel error. Use a custom compare function to avoid
+//
+// This was GT-216 with the ubuntu "nvidia-319" driver package.
+// llvmpipe does not show the same issue.
+//
+bool compareImages(const QImage &ia, const QImage &ib)
+{
+    if (ia.size() != ib.size())
+        qDebug() << "images are of different size" << ia.size() << ib.size();
+    Q_ASSERT(ia.size() == ib.size());
+    Q_ASSERT(ia.format() == ib.format());
+
+    int w = ia.width();
+    int h = ia.height();
+    const int tolerance = 5;
+    for (int y=0; y<h; ++y) {
+        const uint *as= (const uint *) ia.constScanLine(y);
+        const uint *bs= (const uint *) ib.constScanLine(y);
+        for (int x=0; x<w; ++x) {
+            uint a = as[x];
+            uint b = bs[x];
+
+            // No tolerance for error in the alpha.
+            if ((a & 0xff000000) != (b & 0xff000000))
+                return false;
+            if (qAbs(qRed(a) - qRed(b)) > tolerance)
+                return false;
+            if (qAbs(qRed(a) - qRed(b)) > tolerance)
+                return false;
+            if (qAbs(qRed(a) - qRed(b)) > tolerance)
+                return false;
+        }
+    }
+    return true;
+}
 
 void tst_qquickwindow::headless()
 {
@@ -1101,8 +1201,7 @@ void tst_qquickwindow::headless()
 
     // Verify that the visual output is the same
     QImage newContent = window->grabWindow();
-
-    QCOMPARE(originalContent, newContent);
+    QVERIFY(compareImages(newContent, originalContent));
 }
 
 void tst_qquickwindow::noUpdateWhenNothingChanges()
@@ -1495,7 +1594,8 @@ void tst_qquickwindow::requestActivate()
     QVERIFY(windows.at(0)->objectName() == "window2");
 
     window1->show();
-    window1->requestActivate();
+    QVERIFY(QTest::qWaitForWindowExposed(windows.at(0))); //We wait till window 2 comes up
+    window1->requestActivate();                 // and then transfer the focus to window1
 
     QTRY_VERIFY(QGuiApplication::focusWindow() == window1);
     QVERIFY(window1->isActive() == true);
@@ -1521,6 +1621,54 @@ void tst_qquickwindow::requestActivate()
 
     QTRY_VERIFY(QGuiApplication::focusWindow() == windows.at(0));
     QVERIFY(windows.at(0)->isActive());
+    delete window1;
+}
+
+void tst_qquickwindow::testWindowVisibilityOrder()
+{
+    QQmlEngine engine;
+    QQmlComponent component(&engine);
+    component.loadUrl(testFileUrl("windoworder.qml"));
+    QQuickWindow *window1 = qobject_cast<QQuickWindow *>(component.create());
+    QQuickWindow *window2 = window1->property("win2").value<QQuickWindow*>();
+    QQuickWindow *window3 = window1->property("win3").value<QQuickWindow*>();
+    QQuickWindow *window4 = window1->property("win4").value<QQuickWindow*>();
+    QQuickWindow *window5 = window1->property("win5").value<QQuickWindow*>();
+    QVERIFY(window1);
+    QVERIFY(window2);
+    QVERIFY(window3);
+
+    QTest::qWaitForWindowExposed(window3);
+
+    QWindowList windows = QGuiApplication::topLevelWindows();
+    QTRY_COMPARE(windows.size(), 5);
+
+    QVERIFY(window3 == QGuiApplication::focusWindow());
+    QVERIFY(window1->isActive());
+    QVERIFY(window2->isActive());
+    QVERIFY(window3->isActive());
+
+    //Test if window4 is shown 2 seconds after the application startup
+    //with window4 visible window5 (transient child) should also become visible
+    QVERIFY(!window4->isVisible());
+    QVERIFY(!window5->isVisible());
+
+    window4->setVisible(true);
+
+    QTest::qWaitForWindowExposed(window5);
+    QVERIFY(window4->isVisible());
+    QVERIFY(window5->isVisible());
+    window4->hide();
+    window5->hide();
+
+    window3->hide();
+#if defined(Q_OS_OSX)
+    QEXPECT_FAIL("","Focus is not transferred to transient parent on window close (QTBUG-33423)", Continue);
+#endif
+    QTRY_COMPARE(window2 == QGuiApplication::focusWindow(), true);
+
+    window2->hide();
+    QTRY_COMPARE(window1 == QGuiApplication::focusWindow(), true);
 }
 
 void tst_qquickwindow::blockClosing()
@@ -1560,6 +1708,149 @@ void tst_qquickwindow::crashWhenHoverItemDeleted()
     }
 }
 
+// QTBUG-33436
+void tst_qquickwindow::unloadSubWindow()
+{
+    QQmlEngine engine;
+    QQmlComponent component(&engine);
+    component.loadUrl(testFileUrl("unloadSubWindow.qml"));
+    QQuickWindow *window = qobject_cast<QQuickWindow *>(component.create());
+    QVERIFY(window);
+    window->show();
+    QTest::qWaitForWindowExposed(window);
+    QQuickWindow *transient = Q_NULLPTR;
+    QTRY_VERIFY(transient = window->property("transientWindow").value<QQuickWindow*>());
+    QTest::qWaitForWindowExposed(transient);
+
+    // Unload the inner window (in nested Loaders) and make sure it doesn't crash
+    QQuickLoader *loader = window->property("loader1").value<QQuickLoader*>();
+    loader->setActive(false);
+    QTRY_VERIFY(!transient->isVisible());
+}
+
+// QTBUG-32004
+void tst_qquickwindow::qobjectEventFilter_touch()
+{
+    QQuickWindow window;
+
+    window.resize(250, 250);
+    window.setPosition(100, 100);
+    window.show();
+    QVERIFY(QTest::qWaitForWindowExposed(&window));
+
+    TestTouchItem *item = new TestTouchItem(window.contentItem());
+    item->setSize(QSizeF(150, 150));
+
+    EventFilter eventFilter;
+    item->installEventFilter(&eventFilter);
+
+    QPointF pos(10, 10);
+
+    // press single point
+    QTest::touchEvent(&window, touchDevice).press(0, item->mapToScene(pos).toPoint(), &window);
+
+    QCOMPARE(eventFilter.events.count(), 1);
+    QCOMPARE(eventFilter.events.first(), (int)QEvent::TouchBegin);
+}
+
+// QTBUG-32004
+void tst_qquickwindow::qobjectEventFilter_key()
+{
+    QQuickWindow window;
+
+    window.resize(250, 250);
+    window.setPosition(100, 100);
+    window.show();
+    QVERIFY(QTest::qWaitForWindowExposed(&window));
+
+    TestTouchItem *item = new TestTouchItem(window.contentItem());
+    item->setSize(QSizeF(150, 150));
+    item->setFocus(true);
+
+    EventFilter eventFilter;
+    item->installEventFilter(&eventFilter);
+
+    QTest::keyPress(&window, Qt::Key_A);
+
+    // NB: It may also receive some QKeyEvent(ShortcutOverride) which we're not interested in
+    QVERIFY(eventFilter.events.contains((int)QEvent::KeyPress));
+    eventFilter.events.clear();
+
+    QTest::keyRelease(&window, Qt::Key_A);
+
+    QVERIFY(eventFilter.events.contains((int)QEvent::KeyRelease));
+}
+
+// QTBUG-32004
+void tst_qquickwindow::qobjectEventFilter_mouse()
+{
+    QQuickWindow window;
+
+    window.resize(250, 250);
+    window.setPosition(100, 100);
+    window.show();
+    QVERIFY(QTest::qWaitForWindowExposed(&window));
+
+    TestTouchItem *item = new TestTouchItem(window.contentItem());
+    item->setSize(QSizeF(150, 150));
+
+    EventFilter eventFilter;
+    item->installEventFilter(&eventFilter);
+
+    QPoint point = item->mapToScene(QPointF(10, 10)).toPoint();
+    QTest::mousePress(&window, Qt::LeftButton, Qt::NoModifier, point);
+
+    QVERIFY(eventFilter.events.contains((int)QEvent::MouseButtonPress));
+
+    // clean up mouse press state for the next tests
+    QTest::mouseRelease(&window, Qt::LeftButton, Qt::NoModifier, point);
+}
+
+void tst_qquickwindow::animatingSignal()
+{
+    QQuickWindow window;
+    window.setGeometry(100, 100, 300, 200);
+
+    QSignalSpy spy(&window, SIGNAL(afterAnimating()));
+
+    window.show();
+    QTRY_VERIFY(window.isExposed());
+
+    QTRY_VERIFY(spy.count() > 1);
+}
+
+// QTBUG-36938
+void tst_qquickwindow::contentItemSize()
+{
+    QQuickWindow window;
+    QQuickItem *contentItem = window.contentItem();
+    QVERIFY(contentItem);
+    QCOMPARE(QSize(contentItem->width(), contentItem->height()), window.size());
+
+    QSizeF size(300, 200);
+    window.resize(size.toSize());
+    window.show();
+    QVERIFY(QTest::qWaitForWindowExposed(&window));
+
+    QCOMPARE(window.size(), size.toSize());
+    QCOMPARE(QSizeF(contentItem->width(), contentItem->height()), size);
+
+    QQmlEngine engine;
+    QQmlComponent component(&engine);
+    component.setData(QByteArray("import QtQuick 2.1\n Rectangle { anchors.fill: parent }"), QUrl());
+    QQuickItem *rect = qobject_cast<QQuickItem *>(component.create());
+    QVERIFY(rect);
+    rect->setParentItem(window.contentItem());
+    QCOMPARE(QSizeF(rect->width(), rect->height()), size);
+
+    size.transpose();
+    window.resize(size.toSize());
+    QCOMPARE(window.size(), size.toSize());
+    // wait for resize event
+    QTRY_COMPARE(QSizeF(contentItem->width(), contentItem->height()), size);
+    QCOMPARE(QSizeF(rect->width(), rect->height()), size);
+}
+
 QTEST_MAIN(tst_qquickwindow)
 
 #include "tst_qquickwindow.moc"
