@@ -53,12 +53,15 @@
 #endif // QT_GUI_LIB
 
 #include <QQmlApplicationEngine>
+#include <QQmlComponent>
+#include <QDir>
 #include <QFile>
 #include <QFileInfo>
 #include <QRegularExpression>
 #include <QStringList>
 #include <QDebug>
 #include <QStandardPaths>
+#include <QTranslator>
 #include <QtGlobal>
 #include <qqml.h>
 #include <qqmldebug.h>
@@ -261,8 +264,8 @@ void printUsage()
     printf("Any argument ending in .qml will be treated as a QML file to be loaded.\n");
     printf("Any number of QML files can be loaded. They will share the same engine.\n");
     printf("Any argument which is not a recognized option and which does not end in .qml will be ignored.\n");
-    printf("'gui' application type is only available if the QtGui module is avaialble.\n");
-    printf("'widget' application type is only available if the QtWidgets module is avaialble.\n");
+    printf("'gui' application type is only available if the QtGui module is available.\n");
+    printf("'widget' application type is only available if the QtWidgets module is available.\n");
     printf("\n");
     printf("General Options:\n");
     printf("\t-h, -help..................... Print this usage information and exit.\n");
@@ -348,11 +351,7 @@ static void loadDummyDataFiles(QQmlEngine &engine, const QString& directory)
     QStringList list = dir.entryList();
     for (int i = 0; i < list.size(); ++i) {
         QString qml = list.at(i);
-        QFile f(dir.filePath(qml));
-        f.open(QIODevice::ReadOnly);
-        QByteArray data = f.readAll();
-        QQmlComponent comp(&engine);
-        comp.setData(data, QUrl());
+        QQmlComponent comp(&engine, dir.filePath(qml));
         QObject *dummyData = comp.create();
 
         if (comp.isError()) {
@@ -497,6 +496,10 @@ int main(int argc, char *argv[])
     //Load files
     LoadWatcher lw(&e, files.count());
 
+    // Load dummy data before loading QML-files
+    if (!dummyDir.isEmpty() && QFileInfo (dummyDir).isDir())
+        loadDummyDataFiles(e, dummyDir);
+
     foreach (const QString &path, files) {
         //QUrl::fromUserInput doesn't treat no scheme as relative file paths
         QRegularExpression urlRe("[[:word:]]+://.*");
@@ -520,10 +523,6 @@ int main(int argc, char *argv[])
         }
     }
 
-
-    if (!dummyDir.isEmpty() && QFileInfo (dummyDir).isDir())
-        loadDummyDataFiles(e, dummyDir);
-
     return app->exec();
 }
 
